##
# File:  OeMCSS.py
# Date:  29-Nov-2014  J. Westbrook
#
# Updates:
#    6-Jun-2016 jdw general cleanup
#
##
"""
Classes implementing MCSS comparisons --

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import os.path
import sys
import traceback

from openeye.oechem import (
    OEAddExplicitHydrogens,
    OEExprOpts_AtomicNumber,
    OEExprOpts_DefaultAtoms,
    OEExprOpts_DefaultBonds,
    OEExprOpts_ExactAtoms,
    OEExprOpts_ExactBonds,
    OEFloatArray,
    OEMCSMaxAtoms,
    OEMCSSearch,
    OEMCSType_Approximate,
)
from wwpdb.utils.cc_dict_util.timeout.TimeoutMultiProc import timeout
from wwpdb.utils.oe_util.build.OeBuildMol import OeBuildMol


class OeMCSS(object):
    """Perform MCSS comparisons.  Targets can be chemical component identifiers
    or paths to chemical component definition files.  Inputs can be in the the form of pairs,
    lists, and pair lists of chemical component definitions.

    """

    def __init__(self, verbose=True, log=sys.stderr):
        #
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__refId = None
        self.__refmol = None
        self.__refTitle = None
        #
        self.__fitId = None
        self.__fitmol = None
        self.__fitTitle = None
        #
        self.__pairTupleList = []
        #
        self.__minAtomMatchFraction = 0.50
        #
        self.__searchType = "default"
        #
        self.__refFD = {}
        self.__fitFD = {}
        #
        self.__refPath = None
        self.__fitPath = None
        self.__mcss = None

    def setSearchType(self, sType="default"):
        self.__searchType = sType
        return self.__searchType

    def setRefId(self, ccId, title=None, suppressHydrogens=False, cachePath="/data/components/ligand-dict-v3"):
        """Set the query molecule for MCSS comparison using the input chemical component ID.
        It is assumed that the definition for this ID can be obtained from the chemical component
        repository.

        Once the reference molecule is built, the MCSS calculation is initialized.

        A title is optionally provided otherwise the component Id will be used.

        The hydrogen flag can be used to perform the MCSS using only heavy atoms.
        """
        self.__refId = ccId
        ccIdU = ccId.upper()
        self.__refPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")

        cid, self.__refmol, self.__refFD = self.__getCCDefFile(self.__refPath, suppressHydrogens=suppressHydrogens)  # pylint: disable=unused-variable
        #
        # Insert title here -
        if title is not None:
            self.__refmol.SetTitle(title)
            self.__refTitle = title
        else:
            self.__refmol.SetTitle(self.__refId)
            self.__refTitle = None
        #
        #
        self.__setupMCSS(self.__refmol)

    def setRefPath(self, ccPath, title=None, suppressHydrogens=False, type="CC", importType="2D"):  # pylint: disable=redefined-builtin
        """Set the query molecule for MCSS comparison using the input file path.

        The file type is either ['CC'] for a chemical component definition or another file type
        supported by OE toolkit assumed to have a conventional file extension for this type.

        Once the reference molecule is built, the MCSS calculation is initialized.

        A title is optionally provided otherwise the component Id will be used.

        The hydrogen flag can be used to perform the MCSS using only heavy atoms.
        """
        self.__refPath = ccPath
        if type in ["CC"]:
            (self.__refId, self.__refmol, self.__refFD) = self.__getCCDefFile(ccPath, suppressHydrogens=suppressHydrogens)
        else:
            (self.__refId, self.__refmol, self.__refFD) = self.__getMiscFile(ccPath, suppressHydrogens=suppressHydrogens, importType=importType)

        if self.__verbose:
            self.__lfh.write("Derived ref ID     = %s\n" % self.__refId)
            self.__lfh.write("SMILES (stereo)  = %s\n" % self.__refFD["SMILES_STEREO"])
        #
        # Insert title here -
        if title is not None:
            self.__refmol.SetTitle(title)
            self.__refTitle = title
        else:
            self.__refmol.SetTitle(self.__refId)
            self.__refTitle = None

        self.__setupMCSS(self.__refmol)

    def setFitId(self, ccId, title=None, suppressHydrogens=False, cachePath="/data/components/ligand-dict-v3"):
        """Set the ID of the target/library molecule for MCSS comparison."""
        self.__fitId = ccId
        ccIdU = ccId.upper()
        self.__fitPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
        self.__fitId, self.__fitmol, self.__fitFD = self.__getCCDefFile(self.__fitPath, suppressHydrogens=suppressHydrogens)
        if self.__verbose:
            self.__lfh.write("Fit ID             = %s\n" % self.__fitId)
            self.__lfh.write("SMILES (isomeric)  = %s\n" % self.__fitFD["SMILES_STEREO"])
        if title is not None:
            self.__fitmol.SetTitle(title)
            self.__fitTitle = title
        else:
            self.__fitmol.SetTitle(self.__fitId)
            self.__fitTitle = None

    def setFitPath(self, ccPath, title=None, suppressHydrogens=False):
        """Set the path of the target/library molecule for MCSS comparison."""
        (self.__fitId, self.__fitmol, self.__fitFD) = self.__getCCDefFile(ccPath, suppressHydrogens=suppressHydrogens)
        if title is not None:
            self.__fitmol.SetTitle(title)
            self.__fitTitle = title
        else:
            self.__fitmol.SetTitle(self.__fitId)
            self.__fitTitle = None

    def setFitIdList(self, ccIdList, cachePath="/data/components/ligand-dict-v3", suppressHydrogens=False):  # pylint: disable=unused-argument
        """Set the list of IDs to be compared with reference molecule by MCSS.

        From the input ID list build the internal pair list of
        tuples  [(refId,refPath,refTitle,fitId,fitPath,fitTitle),(),...]
        """
        self.__pairTupleList = []
        for ccId in ccIdList:
            refId = self.__refId
            refPath = self.__refPath
            #
            fitId = ccId
            ccIdU = ccId.upper()
            fitPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
            #
            refTitle = refId + "/" + fitId
            fitTitle = fitId + "/" + refId
            self.__pairTupleList.append((refId, refPath, refTitle, fitId, fitPath, fitTitle))

    def setPairIdList(self, pairIdList, cachePath="/data/components/ligand-dict-v3", suppressHydrogens=False):  # pylint: disable=unused-argument
        """Set the list of ID pais to be aligned by MCSS.

        From the input ID list build the internal pair list of
        tuples  [(refId,refPath,refTitle,fitId,fitPath,fitTitle),(),...]
        """

        self.__pairTupleList = []
        for refId, fitId in pairIdList:
            ccIdU = refId.upper()
            refPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
            #
            ccIdU = fitId.upper()
            fitPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
            #
            refTitle = refId + "/" + fitId
            fitTitle = fitId + "/" + refId
            self.__pairTupleList.append((refId, refPath, refTitle, fitId, fitPath, fitTitle))

    def __getCCDefFile(self, ccPath, suppressHydrogens=False):
        """Fetch the molecule definition (ccPath) and build OE molecules
        for comparison and depiction.

        """
        #
        oem = OeBuildMol(verbose=self.__verbose, log=self.__lfh)
        ccId = oem.setChemCompPath(ccPath)
        #
        oem.build2D()

        if self.__verbose:
            self.__lfh.write("+OeMCSS.__getCCDefFile() for %s\n" % ccId)
            self.__lfh.write("  Title              = %s\n" % oem.getTitle())
            self.__lfh.write("  SMILES             = %s\n" % oem.getCanSMILES())
            self.__lfh.write("  SMILES (stereo)    = %s\n" % oem.getIsoSMILES())
            self.__lfh.write("  Formula (Hill)     = %s\n" % oem.getFormula())
            self.__lfh.write("  InChI key          = %s\n" % oem.getInChIKey())
            self.__lfh.write("  InChI              = %s\n" % oem.getInChI())

        fD = {}
        fD = {"Formula": oem.getFormula(), "SMILES": oem.getCanSMILES(), "SMILES_STEREO": oem.getIsoSMILES(), "InChI": oem.getInChI(), "InChIKey": oem.getInChIKey()}

        if suppressHydrogens:
            tMol = oem.getGraphMolSuppressH()
        else:
            tMol = oem.getMol()

        fD["OEMOL"] = tMol

        return (ccId, tMol, fD)

    def __getMiscFile(self, ccPath, suppressHydrogens=False, importType="2D"):
        """Fetch a miscellaneous chemical file (ccPath) and build OE molecules
        for comparison and depiction.

        """
        try:
            oem = OeBuildMol(verbose=self.__verbose, log=self.__lfh)
            if oem.importFile(ccPath, type=importType):
                if self.__verbose:
                    self.__lfh.write("+OeMCSS.__getMiscFile()\n")
                    self.__lfh.write("  Title              = %s\n" % oem.getTitle())
                    self.__lfh.write("  SMILES             = %s\n" % oem.getCanSMILES())
                    self.__lfh.write("  SMILES (stereo)    = %s\n" % oem.getIsoSMILES())
                    self.__lfh.write("  Formula (Hill)     = %s\n" % oem.getFormula())
                    self.__lfh.write("  InChI key          = %s\n" % oem.getInChIKey())
                    self.__lfh.write("  InChI              = %s\n" % oem.getInChI())
            else:
                self.__lfh.write("+OeMCSS.__getMiscFile() Read failed for %s\n" % ccPath)
                return None, None, None
            #
            # oem.build2D()
            ccId = oem.getTitle()
            if suppressHydrogens:
                tMol = oem.getGraphMolSuppressH()
            else:
                tMol = oem.getMol()

            molXyzL = []
            if importType == "3D":
                for ii, atm in enumerate(tMol.GetAtoms()):
                    xyzL = OEFloatArray(3)
                    tMol.GetCoords(atm, xyzL)
                    molXyzL.append((ii, atm.GetIdx(), atm.GetAtomicNum(), atm.GetName(), atm.GetType(), "%.3f" % xyzL[0], "%.3f" % xyzL[1], "%.3f" % xyzL[2]))
            fD = {}
            fD = {
                "Formula": oem.getFormula(),
                "SMILES": oem.getCanSMILES(),
                "SMILES_STEREO": oem.getIsoSMILES(),
                "InChI": oem.getInChI(),
                "InChIKey": oem.getInChIKey(),
                "xyz": molXyzL,
            }

            for ii, atm in enumerate(tMol.GetAtoms()):
                xyzL = OEFloatArray(3)
                tMol.GetCoords(atm, xyzL)
                if self.__verbose:
                    self.__lfh.write("OeMCSS.__getMiscFile - atom  %d %s %s %s %s %r\n" % (ii, atm.GetIdx(), atm.GetAtomicNum(), atm.GetName(), atm.GetType(), xyzL))

            fD["OEMOL"] = tMol
            return (ccId, tMol, fD)
        except Exception as e:  # noqa: F841 pylint: disable=unused-variable
            traceback.print_exc(file=self.__lfh)
            # self.fail()

        return None, None, None

    def __setupMCSS(self, refmol):
        """Internal initialization for the MCSS comparison."""
        #
        if self.__searchType == "default":
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            atomexpr = OEExprOpts_DefaultAtoms
            bondexpr = OEExprOpts_DefaultBonds
        elif self.__searchType == "relaxed":
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            atomexpr = OEExprOpts_AtomicNumber
            bondexpr = 0
        elif self.__searchType == "exact":
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            # self.__mcss = OEMCSSearch(OEMCSType_Exhaustive)
            atomexpr = OEExprOpts_ExactAtoms
            bondexpr = OEExprOpts_ExactBonds
            OEAddExplicitHydrogens(refmol)
        else:
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            atomexpr = OEExprOpts_DefaultAtoms
            bondexpr = OEExprOpts_DefaultBonds
        #
        # atomexpr = OEExprOpts_AtomicNumber|OEExprOpts_EqAromatic
        # bondexpr = 0
        #
        # atomexpr = OEExprOpts_AtomicNumber|OEExprOpts_Aromaticity
        # bondexpr = OEExprOpts_BondOrder|OEExprOpts_EqNotAromatic
        #

        self.__mcss.Init(refmol, atomexpr, bondexpr)

        #
        # self.__mcss.SetMCSFunc(OEMCSMaxBondsCompleteCycles())
        # self.__mcss.SetMCSFunc(OEMCSMaxAtoms())
        #
        # Half of the reference molecule --
        #
        # nAtomsRef=refmol.NumAtoms()
        # self.__mcss.SetMinAtoms(nAtomsRef/2)

    @timeout(30)
    def doAlign(self, suppressHydrogens=False, unique=True, minFrac=1.0):  # pylint: disable=unused-argument
        """Test the MCSS comparison between current reference and fit molecules -
        Return list of corresponding atoms on success or an empty list otherwise.
        """
        atomMap = []
        #
        nAtomsRef = self.__refmol.NumAtoms()
        nAtomsFit = self.__fitmol.NumAtoms()
        minAtoms = int(min(nAtomsRef, nAtomsFit) * minFrac)
        # self.__mcss.SetMinAtoms( int(minAtoms*self.__minAtomMatchFraction) )
        #
        # -------
        self.__mcss.SetMCSFunc(OEMCSMaxAtoms())
        self.__mcss.SetMinAtoms(minAtoms)
        OEAddExplicitHydrogens(self.__refmol)
        OEAddExplicitHydrogens(self.__fitmol)
        #
        # --------
        miter = self.__mcss.Match(self.__fitmol, unique)
        if miter.IsValid():
            match = miter.Target()
            for mAt in match.GetAtoms():
                atomMap.append(
                    (
                        self.__refId,
                        mAt.pattern.GetIdx(),
                        mAt.pattern.GetAtomicNum(),
                        mAt.pattern.GetName(),
                        self.__fitId,
                        mAt.target.GetIdx(),
                        mAt.target.GetAtomicNum(),
                        mAt.target.GetName(),
                    )
                )
        return (nAtomsRef, self.__refFD, nAtomsFit, self.__fitFD, atomMap)



##
# File: OePersist.py
# Date: 22-Feb-2012  John Westbrook
#
# Updates:
#
#      6-Jun-2016  jdw  - general cleanup
##
"""
Class supporting persistant storage of serialized OE molecules.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import shelve
import shutil
import sys
import traceback

try:
    import cPickle as pickle
except ImportError:
    import pickle

from mmcif_utils.persist.LockFile import LockFile


class OePersist(object):
    """ Persistent storage for instances of OE molecules.

    """

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = True
        self.__lfh = log
        #
        self.__inputPdbxfilePath = None
        self.__moleculeList = []

        self.__moleculeNameList = []
        self.__storeObjectIndex = False
        #
        # Parameters to tune lock file management --
        self.__timeoutSeconds = 10
        self.__retrySeconds = 0.2
        # placeholder for LockFile object for open/close methods
        self.__lockObj = None
        self.__isExpired = True
        #
        self.__db = None

    def __reIndex(self):
        """ Rebuild name and type lists from molecule object list.
        """
        #
        self.__moleculeNameList = []
        for molecule in self.__moleculeList:
            self.__moleculeNameList.append(molecule['name'])
        #
        self.__isExpired = False

    def __getMoleculeIndex(self, name):
        try:
            if (self.__isExpired):
                self.__reIndex()
            return self.__moleculeNameList.index(name)
        except:  # noqa: E722 pylint: disable=bare-except
            return None

    def setMoleculeList(self, moleculeList=None):
        """ Initialize molecule data in the internal molecule to be persisted to the data store.
        """
        try:
            self.__moleculeList = moleculeList
            self.__isExpired = True
            if (self.__debug):
                self.__lfh.write("+OePersist.setMoleculeList() - Container list length %d\n" % len(self.__moleculeList))
            return True
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.setMoleculeList() initialization failed %s\n" % str(e))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def appendMoleculeList(self, moleculeList=None):
        """ Append container data to the internal moleculeList to be persisted to the data store.
        """
        try:
            self.__moleculeList.extend(moleculeList)
            self.__isExpired = True
            if (self.__debug):
                self.__lfh.write("+OePersist.appendMoleculeList() - Container list length %d\n" % len(self.__moleculeList))
            return True
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.appendMoleculeList() initialization failed %s\n" % str(e))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def getMolecule(self, moleculeName):
        """ Return the molecule from internal MoleculeList corresponding to the input molecule name.

            **Note that this method is NOT directly fetching data from the internal store.  First recover the
            the store to internal storage before calling this method.
        """
        idx = self.__getMoleculeIndex(moleculeName)
        if idx is not None:
            return self.__moleculeList[idx]
        else:
            return None

    def getMoleculeList(self):
        """ Return the current molecule list.
        """
        return self.__moleculeList

    def getMoleculeNameList(self):
        """ Return the current list of molecule names.
        """
        if (self.__isExpired):
            self.__reIndex()
        return self.__moleculeNameList

    #
    def open(self, dbFileName="my.db", flag='r'):
        """ Open the persistent store with the input database file name and access mode.

            flag = 'r read-only'
                   'c/n new/create'
                   'w read/write'
        """
        try:
            self.__lockObj = LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                                      verbose=self.__verbose, log=self.__lfh)
            self.__lockObj.acquire()
            self.__db = shelve.open(dbFileName, flag=flag, protocol=pickle.HIGHEST_PROTOCOL)
            return True
        except Exception as e:
            if self.__lockObj is not None:
                self.__lockObj.release()
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.open() write failed for file %s %s\n" % (dbFileName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def getStoreMoleculeIndex(self):
        try:
            return self.__db['__index__']
        except:  # noqa: E722 pylint: disable=bare-except
            return []

    def close(self):
        """ Close the persistent store
        """
        try:
            self.__db.close()
            if self.__lockObj is not None:
                self.__lockObj.release()
            self.__db = None
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            if self.__lockObj is not None:
                self.__lockObj.release()
            return False

    def moveStore(self, srcDbFilePath, dstDbFilePath):
        """ Move source store to destination store with destination locking.

            No locking is performed on the source file.
        """
        with LockFile(dstDbFilePath, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                      verbose=self.__verbose, log=self.__lfh) as lf:  # noqa: F841 pylint: disable=unused-variable
            retVal = self.__moveStore(srcDbFilePath, dstDbFilePath)
        return retVal

    def store(self, dbFileName="my.db"):
        """ Store the current molecule list in persistent database.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                      verbose=self.__verbose, log=self.__lfh) as lf:  # noqa: F841 pylint: disable=unused-variable
            retVal = self.__storeShelve(dbFileName)
        return retVal

    def recover(self, dbFileName="my.db"):
        """ Recover the stored state to the current in-memory molecule representation.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                      verbose=self.__verbose, log=self.__lfh) as lf:  # noqa: F841 pylint: disable=unused-variable
            retVal = self.__recoverShelve(dbFileName)
        return retVal

    def getIndex(self, dbFileName="my.db"):
        """ Recover the index of the persistent store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                      verbose=self.__verbose, log=self.__lfh) as lf:  # noqa: F841  pylint: disable=unused-variable
            retVal = self.__indexShelve(dbFileName)
        return retVal

    def updateOneMolecule(self, inputMolecule, dbFileName="my.db"):
        """ Update or append a molecule to an existing store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                      verbose=self.__verbose, log=self.__lfh) as lf:  # noqa: F841 pylint: disable=unused-variable
            retVal = self.__updateMoleculeShelve(dbFileName=dbFileName, inputMolecule=inputMolecule)
        return retVal

    def updateMoleculeList(self, dbFileName="my.db", moleculeList=None):
        """ Update or append the contents of the input molecule list to an existing store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                      verbose=self.__verbose, log=self.__lfh) as lf:  # noqa: F841 pylint: disable=unused-variable
            retVal = self.__updateMoleculeListShelve(dbFileName=dbFileName, updateMoleculeList=moleculeList)
        return retVal

    #
    def fetchOneMolecule(self, dbFileName="my.db", moleculeName=None):
        """  Fetch a single object from a named molecule.  This is atomic operation with respect to the store.
        """
        with LockFile(dbFileName, timeoutSeconds=self.__timeoutSeconds, retrySeconds=self.__retrySeconds,
                      verbose=self.__verbose, log=self.__lfh) as lf:  # noqa: F841  pylint: disable=unused-variable
            retVal = self.__fetchOneMoleculeShelve(dbFileName, moleculeName)
        return retVal

    def fetchMolecule(self, moleculeName=None):
        """ Fetch molecule from an open store.

            Use this method to extract multiple molecules from an open store.
        """
        return self.__fetchMoleculeShelve(moleculeName)

    def __moveStore(self, src, dst):
        """  Internal method to perform file move.
        """
        try:
            shutil.move(src, dst)
            return True
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.__moveStore() move failed for file %s %s\n" % (src, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def __storeShelve(self, dbFileName="my.db"):
        """  Create a new persistent store using the internal molecule list.
        """
        try:
            if (self.__isExpired):
                self.__reIndex()
            db = shelve.open(dbFileName, flag='c', protocol=pickle.HIGHEST_PROTOCOL)
            db['__index__'] = self.__moleculeNameList
            if (self.__debug):
                self.__lfh.write("+OePersist.__storeShelve() - Molecule list length  %d\n" % len(self.__moleculeList))
            for molecule in self.__moleculeList:
                ky = molecule['name']
                db[ky] = molecule
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.__storeShelve() shelve store failed for file %s %s\n" % (dbFileName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def __indexShelve(self, dbFileName="my.db"):
        """  Recover the index of molecules in the persistent store.
        """
        try:
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            self.__moleculeNameList = db['__index__']
            db.close()
            self.__isExpired = False
            return self.__moleculeNameList

        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.__indexShelve() shelve index failed for file %s %s\n" % (dbFileName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return {}

    def __recoverShelve(self, dbFileName="my.db"):
        """  Recover the list of molecules from the persistent store.
        """
        try:
            self.__moleculeList = []
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            self.__moleculeNameList = db['__index__']
            if (self.__debug):
                self.__lfh.write("+OePersist.__recoverShelve() - Molecule name list %r\n" % self.__moleculeNameList)

            for moleculeName in self.__moleculeNameList:
                self.__moleculeList.append(db[moleculeName])

            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.__recoverShelve() shelve recover failed for file %s %s\n" % (dbFileName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def __fetchOneMoleculeShelve(self, dbFileName="my.db", moleculeName=None,):
        """ Recover the molecule from from persistent store corresponding to the
            input molecule name.
        """
        try:
            #
            db = shelve.open(dbFileName, flag='r', protocol=pickle.HIGHEST_PROTOCOL)
            ky = moleculeName
            molecule = db[ky]
            db.close()
            return molecule
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.__fetchOneMoleculeShelve() shelve fetch failed for file %s %s %s\n"
                                 % (dbFileName, moleculeName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return None

    def __fetchMoleculeShelve(self, moleculeName=None):
        """ Recover the molecule from from persistent store corresponding to the
            input molecule name.

            shelve store must be opened from prior call.
        """
        try:
            #
            ky = moleculeName
            molecule = self.__db[ky]
            return molecule
        except Exception as e:
            if (self.__debug):
                self.__lfh.write("+ERROR- OePersist.__fetchObjectShelve() shelve fetch failed %s %s\n"
                                 % (moleculeName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            #
            return None

    def __updateMoleculeShelve(self, dbFileName="my.db", inputMolecule=None):
        """ Update/append the single input molecule object into the persistent store.
        """
        try:
            #
            moleculeName = inputMolecule['name']
            #
            db = shelve.open(dbFileName, flag='w', protocol=pickle.HIGHEST_PROTOCOL)
            #
            # get molecule index -
            #
            moleculeNameList = db['__index__']
            #
            # Modify or append input molecule --
            #
            if moleculeName in moleculeNameList:
                # idx = moleculeNameList.index(moleculeName)
                pass
            else:
                moleculeNameList.append(moleculeName)
            #
            # store updated molecule index lists -
            #
            db['__index__'] = moleculeNameList
            #
            # ------------------------------------------
            #    Update the object contents --
            #
            ky = moleculeName
            db[ky] = inputMolecule
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.__updateObjectShelve() update failed for file %s %s %s\n" % (dbFileName, moleculeName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def __updateMoleculeListShelve(self, dbFileName="my.db", updateMoleculeList=None):
        """ Update/append the contents of the input molecule list into an existing persistent store.
        """
        try:
            db = shelve.open(dbFileName, flag='w', protocol=pickle.HIGHEST_PROTOCOL)
            #
            # get molecule index -
            #
            moleculeNameList = db['__index__']

            for updateMolecule in updateMoleculeList:
                #
                updateMoleculeName = updateMolecule['name']
                #
                # Modify or append input molecule --
                #
                if updateMoleculeName in moleculeNameList:
                    # idx = moleculeNameList.index(updateMoleculeName)
                    pass
                else:
                    moleculeNameList.append(updateMoleculeName)
                #
                # Store updated molecule index lists -
                #
                db['__index__'] = moleculeNameList

                ky = updateMoleculeName
                db[ky] = updateMolecule
            #
            db.close()
            return True
        except Exception as e:
            if (self.__verbose):
                self.__lfh.write("+ERROR- OePersist.__updateMoleculeListShelve() update failed for store %s %s\n" % (dbFileName, str(e)))
            if (self.__debug):
                traceback.print_exc(file=self.__lfh)
            return False

    def __attributePart(self, name):
        i = name.find(".")
        if i == -1:
            return None
        else:
            return name[i + 1:]



##
# File:    OeDescriptorUtils.py
# Author:  jdw
# Date:    22-May-2017
# Version: 0.001
#
# Updates:
#
##
"""
Utilities to standardize chemical descriptors

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import logging

from openeye import oechem

logger = logging.getLogger(__name__)


class OeDescriptorUtils(object):
    ''' Utilities to standardize chemical descriptors
    '''

    def __init__(self):
        pass
        #

    def standardizeSmiles(self, smiles, type="ISOMERIC"):  # pylint: disable=redefined-builtin
        """ Return a standardized SMILES (type) or None
        """
        smilesOut = None
        try:
            mol = oechem.OEGraphMol()
            if (oechem.OEParseSmiles(mol, smiles) == 1):
                oechem.OEAssignAromaticFlags(mol)
                if type == "CANNONICAL":
                    smilesOut = oechem.OECreateCanSmiString(mol)
                elif type == "ISOMERIC":
                    smilesOut = oechem.OECreateIsoSmiString(mol)
            else:
                logger.error("Unable to parse input SMILES '%s'", smiles)

        except Exception as e:
            logger.exception("Error '%s' occured. Arguments %s.", str(e), e.args)

        return smilesOut



##
# File:  OeAlignDepict.py
# Date:  2-Oct-2011  J. Westbrook
#
# Updates:
#  5-Jan-2012 jdw Revise for new OE depiction API
#  5-Mar-2012 jdw refactor
#  7-Nar-2012 jdw adapt to support latest OeBuildMol
#  9-Mar-2012 jdw simplify calling interface
# 10-Jun-2016 jdw add method setFitPathList
#
##
"""
Classes to depict MCS alignments.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import os
import os.path
import sys
import traceback

from openeye.oechem import (
    OEAddExplicitHydrogens,
    OEBlueTint,
    OEExprOpts_AtomicNumber,
    OEExprOpts_DefaultAtoms,
    OEExprOpts_DefaultBonds,
    OEExprOpts_ExactAtoms,
    OEExprOpts_ExactBonds,
    OEFloatArray,
    OEGreenTint,
    OEIsAtomMember,
    OEIsBondMember,
    OEMCSMaxAtoms,
    OEMCSSearch,
    OEMCSType_Approximate,
    OENotAtom,
    OENotBond,
    OEPinkTint,
)
from openeye.oedepict import (
    OE2DMolDisplay,
    OE2DMolDisplayOptions,
    OEAddHighlighting,
    OEAtomStereoStyle_Display_All,
    OEGetMoleculeScale,
    OEHighlightStyle_BallAndStick,
    OEHighlightStyle_Stick,
    OEImage,
    OEImageGrid,
    OEMultiPageImageFile,
    OEPageOrientation_Landscape,
    OEPageOrientation_Portrait,
    OEPageSize_US_Letter,
    OEPrepareAlignedDepiction,
    OEPrepareDepiction,
    OERenderMolecule,
    OEScale_AutoScale,
    OEWriteImage,
    OEWriteMultiPageImage,
)
from wwpdb.utils.cc_dict_util.timeout.TimeoutMultiProc import timeout
from wwpdb.utils.oe_util.build.OeBuildMol import OeBuildMol


class OeDepictMCSAlign(object):

    """Create 2D depictions of MCSS alignments.  Targets can be chemical component identifiers
    or paths to chemical component definition files.  Inputs can be in the the form of pairs,
    lists, and pair lists of chemical component definitions.

    """

    def __init__(self, verbose=True, log=sys.stderr):
        #
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__refId = None
        self.__refmol = None
        self.__refTitle = None
        #
        self.__fitId = None
        self.__fitmol = None
        self.__fitTitle = None
        #
        self.__pairTupleList = []
        #
        self.__minAtomMatchFraction = 0.50
        self.__pageOrientation = "portrait"
        #
        self.__searchType = "default"
        #
        self.__refFD = {}
        self.__fitFD = {}
        #
        self.__refPath = None
        self.__fitPath = None
        self.__mcss = None
        #
        self.__image = None
        self.__grid = None
        self.__opts = None
        self.__gridRows = None
        self.__gridCols = None
        self.__multi = None

    def setSearchType(self, sType="default"):
        self.__searchType = sType
        return self.__searchType

    def setRefId(self, ccId, title=None, suppressHydrogens=False, cachePath="/data/components/ligand-dict-v3"):
        """Set the query molecule for MCSS comparison using the input chemical component ID.
        It is assumed that the definition for this ID can be obtained from the chemical component
        repository.

        Once the reference molecule is built, the MCSS calculation is initialized.

        A title is optionally provided otherwise the component Id will be used.

        The hydrogen flag can be used to perform the MCSS using only heavy atoms.
        """
        self.__refId = ccId
        ccIdU = ccId.upper()
        self.__refPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")

        _id, self.__refmol, self.__refFD = self.__getCCDefFile(self.__refPath, suppressHydrogens=suppressHydrogens)
        #
        # Insert title here -
        if title is not None:
            self.__refmol.SetTitle(title)
            self.__refTitle = title
        else:
            self.__refmol.SetTitle(self.__refId)
            self.__refTitle = None
        #
        #
        OEPrepareDepiction(self.__refmol)
        self.__setupMCSS(self.__refmol)

    def setRefPath(self, ccPath, title=None, suppressHydrogens=False, type="CC", importType="2D"):  # pylint: disable=redefined-builtin
        """Set the query molecule for MCSS comparison using the input file path.

        The file type is either ['CC'] for a chemical component definition or another file type
        supported by OE toolkit assumed to have a conventional file extension for this type.

        Once the reference molecule is built, the MCSS calculation is initialized.

        A title is optionally provided otherwise the component Id will be used.

        The hydrogen flag can be used to perform the MCSS using only heavy atoms.
        """
        self.__refPath = ccPath
        if type in ["CC"]:
            (self.__refId, self.__refmol, self.__refFD) = self.__getCCDefFile(ccPath, suppressHydrogens=suppressHydrogens)
        else:
            (self.__refId, self.__refmol, self.__refFD) = self.__getMiscFile(ccPath, suppressHydrogens=suppressHydrogens, importType=importType)

        if self.__verbose:
            self.__lfh.write("Derived ref ID     = %s\n" % self.__refId)
            self.__lfh.write("SMILES (stereo)  = %s\n" % self.__refFD["SMILES_STEREO"])
        #
        # Insert title here -
        if title is not None:
            self.__refmol.SetTitle(title)
            self.__refTitle = title
        else:
            self.__refmol.SetTitle(self.__refId)
            self.__refTitle = None

        OEPrepareDepiction(self.__refmol)
        self.__setupMCSS(self.__refmol)

    def setFitId(self, ccId, title=None, suppressHydrogens=False, cachePath="/data/components/ligand-dict-v3"):
        """Set the ID of the target/library molecule for MCSS comparison."""
        self.__fitId = ccId
        ccIdU = ccId.upper()
        self.__fitPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
        self.__fitId, self.__fitmol, self.__fitFD = self.__getCCDefFile(self.__fitPath, suppressHydrogens=suppressHydrogens)
        if self.__verbose:
            self.__lfh.write("Fit ID             = %s\n" % self.__fitId)
            self.__lfh.write("SMILES (isomeric)  = %s\n" % self.__fitFD["SMILES_STEREO"])
        if title is not None:
            self.__fitmol.SetTitle(title)
            self.__fitTitle = title
        else:
            self.__fitmol.SetTitle(self.__fitId)
            self.__fitTitle = None

    def setFitPath(self, ccPath, title=None, suppressHydrogens=False):
        """Set the path of the target/library molecule for MCSS comparison."""
        (self.__fitId, self.__fitmol, self.__fitFD) = self.__getCCDefFile(ccPath, suppressHydrogens=suppressHydrogens)
        if title is not None:
            self.__fitmol.SetTitle(title)
            self.__fitTitle = title
        else:
            self.__fitmol.SetTitle(self.__fitId)
            self.__fitTitle = None

    def setFitIdList(self, ccIdList, cachePath="/data/components/ligand-dict-v3", suppressHydrogens=False):  # pylint: disable=unused-argument
        """Set the list of IDs to be compared with reference molecule by MCSS.

        From the input ID list build the internal pair list of
        tuples  [(refId,refPath,refTitle,fitId,fitPath,fitTitle),(),...]
        """
        self.__pairTupleList = []
        for ccId in ccIdList:
            refId = self.__refId
            refPath = self.__refPath
            #
            fitId = ccId
            ccIdU = ccId.upper()
            fitPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
            #
            refTitle = refId + "/" + fitId
            fitTitle = fitId + "/" + refId
            self.__pairTupleList.append((refId, refPath, refTitle, fitId, fitPath, fitTitle))

    def setFitPathList(self, fitPathTupList, suppressHydrogens=False):  # pylint: disable=unused-argument
        """Set the list of paths for the target/library molecules to be compared with reference molecule by MCSS.

        From the input path tuple list build the internal pair list of
        tuples  [(fitId,fitPath,fitTitle),(),...]
        """
        self.__pairTupleList = []
        for fitId, fitPath, fitTitle in fitPathTupList:
            refId = self.__refId
            refPath = self.__refPath
            #
            refTitle = refId + "/" + fitId
            fitTitle = fitId + "/" + refId
            self.__pairTupleList.append((refId, refPath, refTitle, fitId, fitPath, fitTitle))

    def setPairIdList(self, pairIdList, cachePath="/data/components/ligand-dict-v3", suppressHydrogens=False):  # pylint: disable=unused-argument
        """Set the list of ID pais to be aligned by MCSS.

        From the input ID list build the internal pair list of
        tuples  [(refId,refPath,refTitle,fitId,fitPath,fitTitle),(),...]
        """

        self.__pairTupleList = []
        for refId, fitId in pairIdList:
            ccIdU = refId.upper()
            refPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
            #
            ccIdU = fitId.upper()
            fitPath = os.path.join(cachePath, ccIdU[0], ccIdU, ccIdU + ".cif")
            #
            refTitle = refId + "/" + fitId
            fitTitle = fitId + "/" + refId
            self.__pairTupleList.append((refId, refPath, refTitle, fitId, fitPath, fitTitle))

    def __getCCDefFile(self, ccPath, suppressHydrogens=False):
        """Fetch the molecule definition (ccPath) and build OE molecules
        for comparison and depiction.

        """
        #
        oem = OeBuildMol(verbose=self.__verbose, log=self.__lfh)
        ccId = oem.setChemCompPath(ccPath)
        oem.build2D()

        if self.__verbose:
            self.__lfh.write("+OEAlignDepilsct.__getCCDefFile() for %s\n" % ccId)
            self.__lfh.write("  Title              = %s\n" % oem.getTitle())
            self.__lfh.write("  SMILES             = %s\n" % oem.getCanSMILES())
            self.__lfh.write("  SMILES (stereo)    = %s\n" % oem.getIsoSMILES())
            self.__lfh.write("  Formula (Hill)     = %s\n" % oem.getFormula())
            self.__lfh.write("  InChI key          = %s\n" % oem.getInChIKey())
            self.__lfh.write("  InChI              = %s\n" % oem.getInChI())

        fD = {}
        fD = {"Formula": oem.getFormula(), "SMILES": oem.getCanSMILES(), "SMILES_STEREO": oem.getIsoSMILES(), "InChI": oem.getInChI(), "InChIKey": oem.getInChIKey()}

        if suppressHydrogens:
            return (ccId, oem.getGraphMolSuppressH(), fD)
        else:
            return (ccId, oem.getMol(), fD)

    def __getMiscFile(self, ccPath, suppressHydrogens=False, importType="2D"):
        """Fetch a miscellaneous chemical file (ccPath) and build OE molecules
        for comparison and depiction.

        """
        try:
            oem = OeBuildMol(verbose=self.__verbose, log=self.__lfh)
            if oem.importFile(ccPath, type=importType):
                if self.__verbose:
                    self.__lfh.write("+OEAlignDepilsct.__getMiscFile()\n")
                    self.__lfh.write("  Title              = %s\n" % oem.getTitle())
                    self.__lfh.write("  SMILES             = %s\n" % oem.getCanSMILES())
                    self.__lfh.write("  SMILES (stereo)    = %s\n" % oem.getIsoSMILES())
                    self.__lfh.write("  Formula (Hill)     = %s\n" % oem.getFormula())
                    self.__lfh.write("  InChI key          = %s\n" % oem.getInChIKey())
                    self.__lfh.write("  InChI              = %s\n" % oem.getInChI())
            else:
                self.__lfh.write("+OEAlignDepict.__getMiscFile() Read failed for %s\n" % ccPath)
                return None, None, None
            #
            # oem.build2D()
            ccId = oem.getTitle()
            if suppressHydrogens:
                tMol = oem.getGraphMolSuppressH()
            else:
                tMol = oem.getMol()

            molXyzL = []
            if importType == "3D":
                for ii, atm in enumerate(tMol.GetAtoms()):
                    xyzL = OEFloatArray(3)
                    tMol.GetCoords(atm, xyzL)
                    molXyzL.append((ii, atm.GetIdx(), atm.GetAtomicNum(), atm.GetName(), atm.GetType(), "%.3f" % xyzL[0], "%.3f" % xyzL[1], "%.3f" % xyzL[2]))
            fD = {}
            fD = {
                "Formula": oem.getFormula(),
                "SMILES": oem.getCanSMILES(),
                "SMILES_STEREO": oem.getIsoSMILES(),
                "InChI": oem.getInChI(),
                "InChIKey": oem.getInChIKey(),
                "xyz": molXyzL,
            }

            for ii, atm in enumerate(tMol.GetAtoms()):
                xyzL = OEFloatArray(3)
                tMol.GetCoords(atm, xyzL)
                if self.__verbose:
                    self.__lfh.write("OeAlignDepict.__getMiscFile - atom  %d %s %s %s %s %r\n" % (ii, atm.GetIdx(), atm.GetAtomicNum(), atm.GetName(), atm.GetType(), xyzL))

            return (ccId, tMol, fD)
        except:  # noqa: E722 pylint: disable=bare-except
            traceback.print_exc(file=self.__lfh)
            # self.fail()

        return None, None, None

    def __setupMCSS(self, refmol):
        """Internal initialization for the MCSS comparison."""
        #
        if self.__searchType == "default":
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            atomexpr = OEExprOpts_DefaultAtoms
            bondexpr = OEExprOpts_DefaultBonds
        elif self.__searchType == "relaxed":
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            atomexpr = OEExprOpts_AtomicNumber
            bondexpr = 0
        elif self.__searchType == "exact":
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            # self.__mcss = OEMCSSearch(OEMCSType_Exhaustive)
            atomexpr = OEExprOpts_ExactAtoms
            bondexpr = OEExprOpts_ExactBonds
            OEAddExplicitHydrogens(refmol)
        else:
            self.__mcss = OEMCSSearch(OEMCSType_Approximate)
            atomexpr = OEExprOpts_DefaultAtoms
            bondexpr = OEExprOpts_DefaultBonds
        #
        # atomexpr = OEExprOpts_AtomicNumber|OEExprOpts_EqAromatic
        # bondexpr = 0
        #
        # atomexpr = OEExprOpts_AtomicNumber|OEExprOpts_Aromaticity
        # bondexpr = OEExprOpts_BondOrder|OEExprOpts_EqNotAromatic
        #

        self.__mcss.Init(refmol, atomexpr, bondexpr)

        #
        # self.__mcss.SetMCSFunc(OEMCSMaxBondsCompleteCycles())
        # self.__mcss.SetMCSFunc(OEMCSMaxAtoms())
        #
        # Half of the reference molecule --
        #
        # nAtomsRef=refmol.NumAtoms()
        # self.__mcss.SetMinAtoms(nAtomsRef/2)

    @timeout(15)
    def testAlign(self, suppressHydrogens=False, unique=True, minFrac=1.0):  # pylint: disable=unused-argument
        """Test the MCSS comparison between current reference and fit molecules -
        Return list of corresponding atoms on success or an empty list otherwise.
        """
        atomMap = []
        #
        nAtomsRef = self.__refmol.NumAtoms()
        nAtomsFit = self.__fitmol.NumAtoms()
        minAtoms = int(min(nAtomsRef, nAtomsFit) * minFrac)
        # self.__mcss.SetMinAtoms( int(minAtoms*self.__minAtomMatchFraction) )
        #
        # -------
        self.__mcss.SetMCSFunc(OEMCSMaxAtoms())
        self.__mcss.SetMinAtoms(minAtoms)
        OEAddExplicitHydrogens(self.__refmol)
        OEAddExplicitHydrogens(self.__fitmol)
        # --------
        miter = self.__mcss.Match(self.__fitmol, unique)
        if miter.IsValid():
            match = miter.Target()
            for mAt in match.GetAtoms():
                # atomMap.append( (self.__refId,mAt.pattern.GetName(),self.__fitId,mAt.target.GetName() ))
                atomMap.append((self.__refId, mAt.pattern.GetIdx(), mAt.pattern.GetType(), mAt.pattern.GetName(), self.__fitId, mAt.target.GetIdx(), mAt.target.GetName()))

        return atomMap

    @timeout(30)
    def doAlign(self, suppressHydrogens=False, unique=True, minFrac=1.0):  # pylint: disable=unused-argument
        """Test the MCSS comparison between current reference and fit molecules -
        Return list of corresponding atoms on success or an empty list otherwise.
        """
        atomMap = []
        #
        nAtomsRef = self.__refmol.NumAtoms()
        nAtomsFit = self.__fitmol.NumAtoms()
        minAtoms = int(min(nAtomsRef, nAtomsFit) * minFrac)
        # self.__mcss.SetMinAtoms( int(minAtoms*self.__minAtomMatchFraction) )
        #
        # -------
        self.__mcss.SetMCSFunc(OEMCSMaxAtoms())
        self.__mcss.SetMinAtoms(minAtoms)
        OEAddExplicitHydrogens(self.__refmol)
        OEAddExplicitHydrogens(self.__fitmol)
        # --------
        miter = self.__mcss.Match(self.__fitmol, unique)
        if miter.IsValid():
            match = miter.Target()
            for mAt in match.GetAtoms():
                # atomMap.append( (self.__refId,mAt.pattern.GetName(),self.__fitId,mAt.target.GetName() ))
                atomMap.append((self.__refId, mAt.pattern.GetIdx(), mAt.pattern.GetType(), mAt.pattern.GetName(), self.__fitId, mAt.target.GetIdx(), mAt.target.GetName()))

        return (nAtomsRef, self.__refFD["SMILES_STEREO"], nAtomsFit, self.__fitFD["SMILES_STEREO"], atomMap)

    @timeout(30)
    def doAlignAlt(self, suppressHydrogens=False, unique=True, minFrac=1.0):  # pylint: disable=unused-argument
        """Test the MCSS comparison between current reference and fit molecules -
        Return list of corresponding atoms on success or an empty list otherwise.
        """
        atomMap = []
        #
        nAtomsRef = self.__refmol.NumAtoms()
        nAtomsFit = self.__fitmol.NumAtoms()
        minAtoms = int(min(nAtomsRef, nAtomsFit) * minFrac)
        # self.__mcss.SetMinAtoms( int(minAtoms*self.__minAtomMatchFraction) )
        #
        # -------
        self.__mcss.SetMCSFunc(OEMCSMaxAtoms())
        self.__mcss.SetMinAtoms(minAtoms)
        OEAddExplicitHydrogens(self.__refmol)
        OEAddExplicitHydrogens(self.__fitmol)
        #
        # --------
        miter = self.__mcss.Match(self.__fitmol, unique)
        if miter.IsValid():
            match = miter.Target()
            for mAt in match.GetAtoms():
                atomMap.append(
                    (
                        self.__refId,
                        mAt.pattern.GetIdx(),
                        mAt.pattern.GetAtomicNum(),
                        mAt.pattern.GetName(),
                        self.__fitId,
                        mAt.target.GetIdx(),
                        mAt.target.GetAtomicNum(),
                        mAt.target.GetName(),
                    )
                )
        return (nAtomsRef, self.__refFD, nAtomsFit, self.__fitFD, atomMap)

    def __setupImage(self, imageX=400, imageY=400):
        """Internal method to configure a single pair alignment image."""
        #
        self.__image = OEImage(imageX, imageY)
        rows = 1
        cols = 2
        self.__grid = OEImageGrid(self.__image, rows, cols)
        if self.__debug:
            self.__lfh.write("Num columns %d\n" % self.__grid.NumCols())
            self.__lfh.write("Num rows    %d\n" % self.__grid.NumRows())
        self.__opts = OE2DMolDisplayOptions(self.__grid.GetCellWidth(), self.__grid.GetCellHeight(), OEScale_AutoScale)

    def __setupImageMulti(self, gridRows=2, gridCols=2):
        """Internal method to configure a multipage image."""
        #
        self.__gridRows = gridRows
        self.__gridCols = gridCols
        #
        if self.__pageOrientation == "landscape":
            self.__multi = OEMultiPageImageFile(OEPageOrientation_Landscape, OEPageSize_US_Letter)
        else:
            self.__multi = OEMultiPageImageFile(OEPageOrientation_Portrait, OEPageSize_US_Letter)

        # self.__image = self.__multi.NewPage()
        # self.__opts = OE2DMolDisplayOptions()
        self.__newPage()

    def __newPage(self):
        """Internal method to advance to a new page in a multipage configuration."""
        rows = self.__gridRows
        cols = self.__gridCols
        self.__image = self.__multi.NewPage()
        self.__grid = OEImageGrid(self.__image, rows, cols)
        self.__grid.SetCellGap(20)
        self.__grid.SetMargins(20)
        if self.__debug:
            self.__lfh.write("Num columns %d\n" % self.__grid.NumCols())
            self.__lfh.write("Num rows    %d\n" % self.__grid.NumRows())
        self.__opts = OE2DMolDisplayOptions(self.__grid.GetCellWidth(), self.__grid.GetCellHeight(), OEScale_AutoScale)

    def alignPair(self, imagePath="mcs-match.svg", imageX=400, imageY=400):
        """Compare current reference and fit molecules by MCSS.

        Map of corresponding atoms is returned.

        imagePath is the path of the output image.
        """
        atomMap = []
        self.__setupImage(imageX=imageX, imageY=imageY)

        if self.__refTitle is None:
            self.__refTitle = self.__refId + "/" + self.__fitId
        self.__refmol.SetTitle(self.__refTitle)
        OEPrepareDepiction(self.__refmol)

        #
        if self.__fitTitle is None:
            self.__fitTitle = self.__fitId + "/" + self.__refId
        self.__fitmol.SetTitle(self.__fitTitle)
        OEPrepareDepiction(self.__fitmol)

        #
        # opts = OE2DMolDisplayOptions(self.__grid.GetCellWidth(), self.__grid.GetCellHeight(), OEScale_AutoScale)
        #
        refcell = self.__grid.GetCell(1, 1)
        fitcell = self.__grid.GetCell(1, 2)

        self.__opts.SetAtomStereoStyle(OEAtomStereoStyle_Display_All)
        # opts.SetTitleLocation(OETitleLocation_Hidden)

        refscale = OEGetMoleculeScale(self.__refmol, self.__opts)
        fitscale = OEGetMoleculeScale(self.__refmol, self.__opts)
        self.__opts.SetScale(min(refscale, fitscale))

        hstyle = OEHighlightStyle_BallAndStick

        unique = True
        miter = self.__mcss.Match(self.__fitmol, unique)
        if miter.IsValid():
            match = miter.Target()
            OEPrepareAlignedDepiction(self.__fitmol, self.__mcss.GetPattern(), match)

            # Depict reference molecule with MCS highlighting

            refdisp = OE2DMolDisplay(self.__mcss.GetPattern(), self.__opts)
            patoms = OEIsAtomMember(match.GetPatternAtoms())
            OEAddHighlighting(refdisp, OEBlueTint, hstyle, patoms)
            pbonds = OEIsBondMember(match.GetPatternBonds())
            OEAddHighlighting(refdisp, OEBlueTint, hstyle, pbonds)
            OERenderMolecule(refcell, refdisp)

            # Depict fit molecule with MCS highlighting

            fitdisp = OE2DMolDisplay(self.__fitmol, self.__opts)
            tatoms = OEIsAtomMember(match.GetTargetAtoms())
            OEAddHighlighting(fitdisp, OEBlueTint, hstyle, tatoms)
            tbonds = OEIsBondMember(match.GetTargetBonds())
            OEAddHighlighting(fitdisp, OEBlueTint, hstyle, tbonds)
            OERenderMolecule(fitcell, fitdisp)

            for mAt in match.GetAtoms():
                atomMap.append((self.__refId, mAt.pattern.GetName(), self.__fitId, mAt.target.GetName()))

        OEWriteImage(imagePath, self.__image)
        return atomMap

    def alignPairList(
        self,
        imagePath="multi.pdf",
        suppressHydrogens=False,
        gridRows=2,
        gridCols=2,
        optInverseFit=True,
        highLightStyle="stick",
        highLightMatchColor="green",
        highLightNotMatchColor="pink",
    ):
        """Compare molecule pairs in the current __pairTupleList.

        pairTupleList = (refId,refPath,refTitle,fitId,fitPath,fitTitle)

        Map of corresponding atoms is returned.

        Image Output is in multipage layout.
        """
        #
        self.__setupImageMulti(gridRows=gridRows, gridCols=gridCols)
        #
        atomMap = []
        #
        numRows = self.__grid.NumRows()
        rowIdx = 0
        for (refId, refPath, refTitle, fitId, fitPath, fitTitle) in self.__pairTupleList:
            self.setRefPath(refPath, title=refTitle, suppressHydrogens=suppressHydrogens)
            OEPrepareDepiction(self.__refmol)

            #
            _tId, fitmol, _fitFD = self.__getCCDefFile(fitPath, suppressHydrogens=suppressHydrogens)
            fitmol.SetTitle(fitTitle)
            #
            # self.__minAtomMatchFraction %  of the smaller molecule --
            #
            nAtomsRef = self.__refmol.NumAtoms()
            nAtomsFit = fitmol.NumAtoms()
            minAtoms = min(nAtomsRef, nAtomsFit)
            mcssMinAtoms = int(minAtoms * self.__minAtomMatchFraction)
            self.__mcss.SetMinAtoms(mcssMinAtoms)

            OEPrepareDepiction(fitmol)
            rowIdx += 1
            if rowIdx > numRows:
                self.__newPage()
                rowIdx = 1
            refcell = self.__grid.GetCell(rowIdx, 1)
            fitcell = self.__grid.GetCell(rowIdx, 2)

            self.__opts.SetAtomStereoStyle(OEAtomStereoStyle_Display_All)
            # opts.SetTitleLocation(OETitleLocation_Hidden)

            refscale = OEGetMoleculeScale(self.__refmol, self.__opts)
            fitscale = OEGetMoleculeScale(self.__refmol, self.__opts)
            self.__opts.SetScale(min(refscale, fitscale))

            if highLightStyle == "ballAndStick":
                hstyle = OEHighlightStyle_BallAndStick
            elif highLightStyle == "stick":
                hstyle = OEHighlightStyle_Stick
            else:
                hstyle = OEHighlightStyle_BallAndStick

            if highLightMatchColor == "blue":
                myHighLightMatchColor = OEBlueTint
            elif highLightMatchColor == "green":
                myHighLightMatchColor = OEGreenTint
            elif highLightMatchColor == "pink":
                myHighLightMatchColor = OEPinkTint
            else:
                myHighLightMatchColor = OEBlueTint

            if highLightNotMatchColor == "blue":
                myHighLightNotMatchColor = OEBlueTint
            elif highLightNotMatchColor == "green":
                myHighLightNotMatchColor = OEGreenTint
            elif highLightNotMatchColor == "pink":
                myHighLightNotMatchColor = OEPinkTint
            else:
                myHighLightNotMatchColor = OEBlueTint

            optInverseFit = True

            self.__lfh.write("+OeAlignDepict.alignPairList refId %s fitId %s nAtomsRef %d nAtomsFit %s mcssMinAtoms %d\n" % (refId, fitId, nAtomsRef, nAtomsFit, mcssMinAtoms))
            unique = True

            miter = self.__mcss.Match(fitmol, unique)
            if miter.IsValid():
                match = miter.Target()
                OEPrepareAlignedDepiction(fitmol, self.__mcss.GetPattern(), match)

                # Depict reference molecule with MCS highlighting

                refdisp = OE2DMolDisplay(self.__mcss.GetPattern(), self.__opts)
                patoms = OEIsAtomMember(match.GetPatternAtoms())
                OEAddHighlighting(refdisp, myHighLightMatchColor, hstyle, patoms)
                pbonds = OEIsBondMember(match.GetPatternBonds())
                OEAddHighlighting(refdisp, myHighLightMatchColor, hstyle, pbonds)
                OERenderMolecule(refcell, refdisp)

                # Depict fit molecule with MCS highlighting
                if optInverseFit:
                    fitdisp = OE2DMolDisplay(fitmol, self.__opts)
                    tatoms = OEIsAtomMember(match.GetTargetAtoms())
                    OEAddHighlighting(fitdisp, myHighLightNotMatchColor, hstyle, OENotAtom(tatoms))
                    tbonds = OEIsBondMember(match.GetTargetBonds())
                    OEAddHighlighting(fitdisp, myHighLightNotMatchColor, hstyle, OENotBond(tbonds))
                    OERenderMolecule(fitcell, fitdisp)
                else:
                    fitdisp = OE2DMolDisplay(fitmol, self.__opts)
                    tatoms = OEIsAtomMember(match.GetTargetAtoms())
                    OEAddHighlighting(fitdisp, myHighLightNotMatchColor, hstyle, tatoms)
                    tbonds = OEIsBondMember(match.GetTargetBonds())
                    OEAddHighlighting(fitdisp, myHighLightNotMatchColor, hstyle, tbonds)
                    OERenderMolecule(fitcell, fitdisp)

                for mAt in match.GetAtoms():
                    atomMap.append((self.__refId, mAt.pattern.GetName(), fitId, mAt.target.GetName()))

        OEWriteMultiPageImage(imagePath, self.__multi)
        return atomMap



##
# File:    PdbxBuildChemComp.py
# Author:  jdw
# Date:    9-Jun-2016
# Version: 0.001
#
# Updates:
#
#
##
"""
Utilities to build chemical component definitions from OE molecule objects.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import sys
import time
import traceback

from mmcif_utils.style.ChemCompCategoryStyle import ChemCompCategoryStyle
from mmcif_utils.style.PdbxStyleIoUtil import PdbxStyleIoUtil
from openeye.oechem import (OECalculateMolecularWeight,
                            OECIPAtomStereo_NotStereo, OECIPAtomStereo_R,
                            OECIPAtomStereo_S, OECIPAtomStereo_UnspecStereo,
                            OECIPBondStereo_E, OECIPBondStereo_NotStereo,
                            OECIPBondStereo_UnspecStereo, OECIPBondStereo_Z,
                            OECreateCanSmiString, OECreateInChI,
                            OECreateInChIKey, OECreateIsoSmiString,
                            OEGetAtomicSymbol, OEMol, OEMolecularFormula,
                            OEPerceiveCIPStereo, OEWriteConstMolecule,
                            OEWriteMolecule, oemolostream)


class PdbxBuildChemComp(PdbxStyleIoUtil):
    ''' Methods to build chemical component definitions from OE molecule objects.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxBuildChemComp, self).__init__(styleObject=ChemCompCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__topCachePath = None
        self.__ccU = None
        self.__filePath = None

    def getCategory(self, catName='chem_comp'):
        return self.getItemDictList(catName)

    def getChemCompCategory(self, catName='chem_comp'):
        return self.getItemDictList(catName)

    def getChemCompDict(self):
        return self.getItemDictList(catName='chem_comp')

    def getBondList(self):
        return self.getRowDataList(catName='chem_comp_bond')

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)

    #
    def setOeMol(self, oeMol, ccId, name=None, missingModelXyz=True):
        ccIdU = str(ccId).strip().upper()
        self.newContainer(containerName=ccIdU, overWrite=True)
        self.setContainer(containerName=ccIdU)
        #
        self.newCategory('chem_comp', container=None, overWrite=True)
        #
        formula = OEMolecularFormula(oeMol)
        fCharge = self.__getFormalCharge(oeMol)
        fWeight = OECalculateMolecularWeight(oeMol)
        # JDW
        rowD = self.__makeChemCompCategory(ccIdU, name=name, formula=formula, charge=fCharge, fW=fWeight, site='RCSB', missingModelXyz=missingModelXyz)
        self.updateRowByAttribute(rowD, 'chem_comp', iRow=0)
        #
        self.newCategory('chem_comp_atom', container=None, overWrite=True)
        rowDL = self.__makeChemCompAtomCategory(ccIdU, oeMol)
        for ii, rowD in enumerate(rowDL):
            self.updateRowByAttribute(rowD, 'chem_comp_atom', iRow=ii)
        #
        self.newCategory('chem_comp_bond', container=None, overWrite=True)
        rowDL = self.__makeChemCompBondCategory(ccIdU, oeMol)
        for ii, rowD in enumerate(rowDL):
            self.updateRowByAttribute(rowD, 'chem_comp_bond', iRow=ii)
        #
        self.newCategory('pdbx_chem_comp_descriptor', container=None, overWrite=True)
        rowDL = self.__makeChemCompDescriptorCategory(ccIdU, oeMol)
        for ii, rowD in enumerate(rowDL):
            self.updateRowByAttribute(rowD, 'pdbx_chem_comp_descriptor', iRow=ii)
        #
        self.newCategory('pdbx_chem_comp_audit', container=None, overWrite=True)
        rowD = self.__makeChemCompAuditRow(ccIdU)
        self.updateRowByAttribute(rowD, 'pdbx_chem_comp_audit', iRow=0)
        #
        #

    def writeOther(self, oeMol, filePath, title='None', constantMol=False):
        try:
            ofs = oemolostream()
            ofs.open(filePath)
            myMol = OEMol(oeMol)
            myMol.SetTitle(title)
            self.__lfh.write("+PdbxBuildChemComp.writeOther writing %s title %s\n" % (filePath, myMol.GetTitle()))
            if constantMol:
                OEWriteConstMolecule(ofs, myMol)
            else:
                OEWriteMolecule(ofs, myMol)
            return True
        except Exception as e:
            self.__lfh.write("+PdbxBuildChemComp.writeOther FAILING %s\n" % str(e))
            traceback.print_exc(file=self.__lfh)
        return False

    def __getFormalCharge(self, oeMol):
        fCharge = 0
        for atom in oeMol.GetAtoms():
            fCharge += atom.GetFormalCharge()
        return fCharge

    def __makeChemCompAuditRow(self, ccId, action='CREATE', date=None, processingSite='RCSB', annotator='?', details='?'):
        '''
            loop_
            _pdbx_chem_comp_audit.comp_id
            _pdbx_chem_comp_audit.action_type
            _pdbx_chem_comp_audit.date
            _pdbx_chem_comp_audit.processing_site
            _pdbx_chem_comp_audit.annotator
            _pdbx_chem_comp_audit.details
            ARG "Create component"  1999-07-08 RCSB ? ?

        '''
        aRow = {}
        aRow['comp_id'] = ccId
        aRow['action_type'] = action
        if date is None:
            date = time.strftime("%Y-%m-%d", time.localtime())
        aRow['date'] = date
        aRow['processing_site'] = processingSite
        aRow['annotator'] = annotator
        aRow['details'] = details
        #
        return aRow

    def __makeChemCompDescriptorCategory(self, ccId, oeMol):
        '''
            loop_
            _pdbx_chem_comp_descriptor.comp_id
            _pdbx_chem_comp_descriptor.type
            _pdbx_chem_comp_descriptor.program
            _pdbx_chem_comp_descriptor.program_version
            _pdbx_chem_comp_descriptor.descriptor
            ARG SMILES           ACDLabs              10.04 "O=C(O)C(N)CCCN\\C(=[NH2+])N"
            ARG SMILES_CANONICAL CACTVS               3.341 "N[C@@H](CCCNC(N)=[NH2+])C(O)=O"
            ARG SMILES           CACTVS               3.341 "N[CH](CCCNC(N)=[NH2+])C(O)=O"
            ARG SMILES_CANONICAL "OpenEye OEToolkits" 1.5.0 "C(C[C@@H](C(=O)O)N)CNC(=[NH2+])N"
            ARG SMILES           "OpenEye OEToolkits" 1.5.0 "C(CC(C(=O)O)N)CNC(=[NH2+])N"
            ARG InChI            InChI                1.03  "InChI=1S/C6H14N4O2/c7-4(5(11)12)2-1-3-1..... "
            ARG InChIKey         InChI                1.03  ODKSFYDXXFIFQN-BYPYZUCNSA-O
            #
        '''
        rowL = []
        #
        aRow = {}
        aRow['comp_id'] = ccId
        aRow['type'] = 'SMILES_CANONICAL'
        aRow['program'] = "OpenEye OEToolkits"
        aRow['program_version'] = '2016.2'
        aRow['descriptor'] = OECreateIsoSmiString(oeMol)
        rowL.append(aRow)
        #
        aRow = {}
        aRow['comp_id'] = ccId
        aRow['type'] = 'SMILES'
        aRow['program'] = "OpenEye OEToolkits"
        aRow['program_version'] = '2016.2'
        aRow['descriptor'] = OECreateCanSmiString(oeMol)
        rowL.append(aRow)
        #
        aRow = {}
        aRow['comp_id'] = ccId
        aRow['type'] = 'InChI'
        aRow['program'] = "OpenEye OEToolkits"
        aRow['program_version'] = '2016.2'
        aRow['descriptor'] = OECreateInChI(oeMol)
        rowL.append(aRow)
        #
        aRow = {}
        aRow['comp_id'] = ccId
        aRow['type'] = 'InChIKey'
        aRow['program'] = "OpenEye OEToolkits"
        aRow['program_version'] = '2016.2'
        aRow['descriptor'] = OECreateInChIKey(oeMol)
        rowL.append(aRow)
        #
        return rowL

    def __makeChemCompCategory(self, ccId, name=None, formula=None, charge=None, fW=None, site='RCSB', missingModelXyz=False):
        #
        lt = time.strftime("%Y-%m-%d", time.localtime())
        ccRow = {}
        ccRow['id'] = ccId
        if name is not None:
            ccRow['name'] = name
        else:
            ccRow['name'] = '?'
        ccRow['type'] = 'NON-POLYMER'
        ccRow['pdbx_type'] = '?'
        if formula is not None:
            ccRow['formula'] = formula
        else:
            ccRow['formula'] = '?'
        ccRow['mon_nstd_parent_comp_id'] = '?'
        ccRow['pdbx_synonyms'] = '?'
        if charge is not None:
            ccRow['pdbx_formal_charge'] = charge
        else:
            ccRow['pdbx_formal_charge'] = '?'
        ccRow['pdbx_ambiguous_flag'] = 'N'
        ccRow['pdbx_initial_date'] = lt
        ccRow['pdbx_modified_date'] = lt
        ccRow['pdbx_release_status'] = 'HOLD'
        ccRow['pdbx_replaced_by'] = '?'
        ccRow['pdbx_replaces'] = '?'
        if fW is not None:
            ccRow['formula_weight'] = fW
        else:
            ccRow['formula_weight'] = '?'
        ccRow['one_letter_code'] = '?'
        tlc = ccId.split("_")[0]
        ccRow['three_letter_code'] = tlc
        ccRow['pdbx_model_coordinates_details'] = '?'
        ccRow['pdbx_ideal_coordinates_details'] = '?'
        if missingModelXyz:
            ccRow['pdbx_model_coordinates_missing_flag'] = 'Y'
        else:
            ccRow['pdbx_model_coordinates_missing_flag'] = 'N'
        ccRow['pdbx_model_coordinates_db_code'] = '?'
        ccRow['pdbx_processing_site'] = site
        ccRow['pdbx_subcomponent_list'] = '?'
        return ccRow

    def __makeChemCompAtomCategory(self, ccId, oeMol):
        """ Populate elements of chemical component definition for atoms and bonds.
        """
        #
        idCode = ccId
        #
        rowL = []
        for ii, atom in enumerate(oeMol.GetAtoms()):
            #
            atRow = {}
            atRow['comp_id'] = idCode
            (x, y, z) = oeMol.GetCoords(atom)
            # pdbx_model_Cartn_x_ideal
            atRow['pdbx_model_Cartn_x_ideal'] = '%0.3f' % x
            atRow['pdbx_model_Cartn_y_ideal'] = '%0.3f' % y
            atRow['pdbx_model_Cartn_z_ideal'] = '%0.3f' % z
            atRow['atom_id'] = atom.GetName().strip()
            atRow['alt_atom_id'] = atom.GetName().strip()
            atRow['type_symbol'] = OEGetAtomicSymbol(atom.GetAtomicNum())
            atRow['charge'] = atom.GetFormalCharge()
            if atom.GetStringData('pdbx_leaving_atom_flag') in ['Y', 'N']:
                atRow['pdbx_leaving_atom_flag'] = atom.GetStringData('pdbx_leaving_atom_flag')
            else:
                atRow['pdbx_leaving_atom_flag'] = 'N'
            if len(atRow['atom_id']) > 3 or len(atRow['type_symbol']) == 2:
                atRow['pdbx_align'] = 0
            else:
                atRow['pdbx_align'] = 1

            if atom.IsAromatic():
                atRow['pdbx_aromatic_flag'] = 'Y'
            else:
                atRow['pdbx_aromatic_flag'] = 'N'
            # oeSt = OEGetCIPStereo(mol, atom)
            oeSt = None
            # if atom.IsChiral():
            cip = OEPerceiveCIPStereo(oeMol, atom)
            if atom.HasStereoSpecified():
                if cip == OECIPAtomStereo_S:
                    oeSt = "S"
                if cip == OECIPAtomStereo_R:
                    oeSt = "R"
                if cip == OECIPAtomStereo_NotStereo:
                    oeSt = None
                if cip == OECIPAtomStereo_UnspecStereo:
                    oeSt = "U"
                # oeSt = OEGetCIPStereo(oeMol, atom)
                # oeSt = atom.GetCIPStereo()
            if (oeSt is not None and (len(oeSt) > 0) and (oeSt == "R" or oeSt == "S")):
                atRow['pdbx_stereo_config'] = oeSt
            else:
                atRow['pdbx_stereo_config'] = 'N'
            atRow['pdbx_component_atom_id'] = atom.GetName().strip()
            atRow['pdbx_component_comp_id'] = idCode
            atRow['pdbx_ordinal'] = str(ii + 1)
            rowL.append(atRow)
        #
        return rowL

    def __makeChemCompBondCategory(self, ccId, oeMol):
        idCode = ccId
        rowL = []
        iBondD = {1: "SING", 2: "DOUB", 3: "TRIP", 4: "QUAD", 5: "AROM", 0: "DELO"}

        for ii, bond in enumerate(oeMol.GetBonds()):
            oeOrder = str(bond.GetOrder()).upper()
            b1 = bond.GetBgn().GetName().strip()
            b2 = bond.GetEnd().GetName().strip()
            if ((b1 is None) or (len(b1) < 1)):
                continue
            if ((b2 is None) or (len(b1) < 1)):
                continue
            if (int(oeOrder) in iBondD):
                bndRow = {}
                bndRow['comp_id'] = idCode
                bndRow['atom_id_1'] = b1
                bndRow['atom_id_2'] = b2
                bndRow['value_order'] = iBondD[int(oeOrder)]

                if bond.IsAromatic():
                    bndRow['pdbx_aromatic_flag'] = 'Y'
                else:
                    bndRow['pdbx_aromatic_flag'] = 'N'

                oeSt = None
                if (bond.GetOrder() == 2):
                    cip = OEPerceiveCIPStereo(oeMol, bond)
                    if bond.HasStereoSpecified():
                        if cip == OECIPBondStereo_E:
                            oeSt = "E"
                        if cip == OECIPBondStereo_Z:
                            oeSt = "Z"
                        if cip == OECIPBondStereo_NotStereo:
                            oeSt = None
                        if cip == OECIPBondStereo_UnspecStereo:
                            oeSt = None
                if (oeSt is not None and (len(oeSt) > 0) and (oeSt == "E" or oeSt == "Z")):
                    bndRow['pdbx_stereo_config'] = oeSt
                else:
                    bndRow['pdbx_stereo_config'] = 'N'

                bndRow['pdbx_ordinal'] = str(ii + 1)
                rowL.append(bndRow)
        return rowL



##
# File:  OeDepict.py
# Date:  3-May-2013  J. Westbrook
#
# Updates:
# 05-May-2013  jdw strip out IO methods -
# 21-Jan-2014  rps setGridOptions updated to support option for explicitly setting cellBorders
#  6-Jun-2016  jdw general cleanup
# 12-Jun-2016  jdw updated display parameter defaults
#
##
"""
Classes to build 2D OE molecule depictions from ChemComp definitions.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import sys

from openeye.oechem import OEBlack, OEDarkBlue, OEDarkGreen
from openeye.oedepict import (
    OE2DMolDisplay,
    OE2DMolDisplayOptions,
    OEAtomStereoStyle_Display_All,
    OEBlackPen,
    OEDisplayAtomIdx,
    OEDisplayAtomPropBase,
    OEDisplayBondIdx,
    OEDrawBorder,
    OEFill_On,
    OEFont,
    OEImage,
    OEImageGrid,
    OEMultiPageImageFile,
    OEPageOrientation_Landscape,
    OEPageOrientation_Portrait,
    OEPageSize_US_Letter,
    OEPen,
    OEPrepareDepiction,
    OERenderMolecule,
    OEScale_AutoScale,
    OEWriteImage,
    OEWriteMultiPageImage,
)


class LabelAtoms(OEDisplayAtomPropBase):
    def __init__(self):
        OEDisplayAtomPropBase.__init__(self)

    def __call__(self, atom):
        return atom.GetName()

    def CreateCopy(self):
        # __disown__ is required to allow C++ to take
        # ownsership of this object and its memory
        copy = LabelAtoms()
        return copy.__disown__()


class OeDepictBase(object):

    """Base class for 2D depictions in single and multi-page format."""

    def __init__(self, verbose=True, log=sys.stderr):
        super(OeDepictBase, self).__init__()
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self._molTitleList = []
        self._opts = None
        #
        # internal dictionary of display parameters -
        #
        self._params = {
            "imageSizeX": 500,
            "imageSizeY": 500,
            "cellBorders": True,
            "suppressHydrogens": False,
            "labelAtomName": False,
            "labelAtomCIPStereo": False,
            "labelAtomIndex": False,
            "labelBondIndex": False,
            "bondDisplayWidth": None,
            "gridRows": 2,
            "gridCols": 2,
            "cellGap": 5,
            "cellMargin": 10,
            "pageOrientation": "landscape",
            "highlightStyleRef": "none",
            "highlightStyleFit": "ballAndStick",
            "highLightMatchColorRef": "blue",
            "highLightMatchColorFit": "blue",
            "highLightNotMatchColorRef": "pink",
            "highLightNotMatchColorFit": "pink",
        }

    def setDisplayOptions(self, **kwargs):
        self._params.update(kwargs)

    def setGridOptions(self, rows=1, cols=1, cellBorders=True):
        self._params["gridRows"] = rows
        self._params["gridCols"] = cols
        self._params["cellBorders"] = cellBorders

    def setMolTitleList(self, oeMolTitleList):
        """Set the list of OE Mols to be depicted as a list of tuples containing
        [(ccId,oeMol,titleString),(ccId,oeMol,titleString),...]
        """
        self._molTitleList = oeMolTitleList

    def _assignDisplayOptions(self):
        if self._params["labelAtomCIPStereo"]:
            self._opts.SetAtomStereoStyle(OEAtomStereoStyle_Display_All)

        if self._params["labelAtomIndex"]:
            self._opts.SetAtomPropertyFunctor(OEDisplayAtomIdx())
            self._opts.SetAtomPropLabelFont(OEFont(OEDarkGreen))
            self._opts.SetAtomPropLabelFontScale(0.75)

        if self._params["labelBondIndex"]:
            self._opts.SetBondPropertyFunctor(OEDisplayBondIdx())
            self._opts.SetBondPropLabelFont(OEFont(OEDarkBlue))

        if self._params["labelAtomName"]:
            atomlabel = LabelAtoms()
            self._opts.SetAtomPropertyFunctor(atomlabel)
            self._opts.SetAtomPropLabelFont(OEFont(OEDarkGreen))
            self._opts.SetAtomPropLabelFontScale(0.75)

        if self._params["bondDisplayWidth"] is not None:
            pen = OEPen(OEBlack, OEBlack, OEFill_On, self._params["bondDisplayWidth"])
            self._opts.SetDefaultBondPen(pen)
            # remove for the moment.  not supported on all platforms
            # self._opts.SetBondWidthScaling(False)
        #
        # 5.0 is minimum size -
        self._opts.SetTitleHeight(5.0)
        #


class OeDepictMultiPage(OeDepictBase):

    """Create 2D depictions in multipage format from a list of OE molecules and title strings"""

    def __init__(self, verbose=True, log=sys.stderr, useTitle=True):
        super(OeDepictMultiPage, self).__init__(verbose=verbose, log=log)
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        self.__useTitle = useTitle
        #
        self.__multi = None
        self.__image = None

    def __setupImage(self):
        if self._params["pageOrientation"] == "landscape":
            self.__multi = OEMultiPageImageFile(OEPageOrientation_Landscape, OEPageSize_US_Letter)
        else:
            self.__multi = OEMultiPageImageFile(OEPageOrientation_Portrait, OEPageSize_US_Letter)
        self.__image = self.__multi.NewPage()
        self._opts = OE2DMolDisplayOptions()

    def prepare(self):
        self.__setupImage()
        rows = self._params["gridRows"]
        cols = self._params["gridCols"]
        grid = OEImageGrid(self.__image, rows, cols)

        citer = grid.GetCells()

        for _ccId, oeMol, title in self._molTitleList:
            if not citer.IsValid():
                # go to next page
                self.__image = self.__multi.NewPage()
                grid = OEImageGrid(self.__image, rows, cols)
                grid.SetCellGap(self._params["cellGap"])
                grid.SetMargins(self._params["cellMargin"])
                citer = grid.GetCells()

            cell = citer.Target()
            #
            if self._params["suppressHydrogens"]:
                mol = oeMol.getGraphMolSuppressH()
            else:
                mol = oeMol.getMol()

            if self.__useTitle:
                mol.SetTitle(title)
                self._opts.SetTitleHeight(5.0)
            else:
                mol.SetTitle("")
            #
            #
            OEPrepareDepiction(mol)
            self._opts.SetDimensions(cell.GetWidth(), cell.GetHeight(), OEScale_AutoScale)
            self._assignDisplayOptions()

            disp = OE2DMolDisplay(mol, self._opts)
            OERenderMolecule(cell, disp)
            OEDrawBorder(cell, OEPen(OEBlackPen))

            citer.Next()

    def write(self, imagePath):
        OEWriteMultiPageImage(imagePath, self.__multi)


class OeDepict(OeDepictBase):

    """Create 2D depictions in single-page format from a list of OE molecules & title strings"""

    def __init__(self, verbose=True, log=sys.stderr, useTitle=True):
        super(OeDepict, self).__init__(verbose=verbose, log=log)
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        self.__useTitle = useTitle
        #
        self.__image = None
        self.__grid = None

    def __setupImage(self):
        """Internal method to configure a single page image."""
        #
        self.__image = OEImage(self._params["imageSizeX"], self._params["imageSizeY"])
        self.__grid = OEImageGrid(self.__image, self._params["gridRows"], self._params["gridCols"])
        self.__grid.SetCellGap(self._params["cellGap"])
        self.__grid.SetMargins(self._params["cellMargin"])
        self._opts = OE2DMolDisplayOptions(self.__grid.GetCellWidth(), self.__grid.GetCellHeight(), OEScale_AutoScale)
        #
        if self.__debug:
            self.__lfh.write("Num columns %d\n" % self.__grid.NumCols())
            self.__lfh.write("Num rows    %d\n" % self.__grid.NumRows())

    def prepare(self):
        self.__setupImage()
        # We convert to list as may come in as a python 3 zipobject with is not indexable
        mollist = list(self._molTitleList)
        for idx, cell in enumerate(self.__grid.GetCells()):
            _ccId, oeMol, title = mollist[idx]
            #
            if self._params["suppressHydrogens"]:
                mol = oeMol.getGraphMolSuppressH()
            else:
                mol = oeMol.getMol()
            if self.__useTitle:
                mol.SetTitle(title)
                self._opts.SetTitleHeight(5.0)
            else:
                mol.SetTitle("")
            #
            #
            OEPrepareDepiction(mol)
            self._opts.SetDimensions(cell.GetWidth(), cell.GetHeight(), OEScale_AutoScale)

            self._assignDisplayOptions()

            disp = OE2DMolDisplay(mol, self._opts)
            OERenderMolecule(cell, disp)
            if self._params["cellBorders"]:
                OEDrawBorder(cell, OEPen(OEBlackPen))

    def write(self, imagePath):
        OEWriteImage(imagePath, self.__image)



#!/usr/bin/env python
# @ <SNIPPET-MCS-ALIGN>
from openeye.oechem import (
    OEExprOpts_DefaultAtoms,
    OEExprOpts_DefaultBonds,
    OEGraphMol,
    OEIsAtomMember,
    OEIsBondMember,
    OEMCSMaxBondsCompleteCycles,
    OEMCSSearch,
    OEMCSType_Approximate,
    OENotAtom,
    OENotBond,
    OEParseSmiles,
    OEPinkTint,
)
from openeye.oedepict import (
    OE2DMolDisplay,
    OE2DMolDisplayOptions,
    OEAddHighlighting,
    OEGetMoleculeScale,
    OEHighlightStyle_BallAndStick,
    OEImage,
    OEImageGrid,
    OEPrepareAlignedDepiction,
    OEPrepareDepiction,
    OERenderMolecule,
    OEScale_AutoScale,
    OETitleLocation_Hidden,
    OEWriteImage,
)

refmol = OEGraphMol()
OEParseSmiles(refmol, "c1cc(c2cc(cnc2c1)CCCO)C(=O)CCO")
OEPrepareDepiction(refmol)

fitmol = OEGraphMol()
OEParseSmiles(fitmol, "c1cc2ccc(cc2c(c1)C(=O)O)CCO")
OEPrepareDepiction(fitmol)

mcss = OEMCSSearch(OEMCSType_Approximate)
atomexpr = OEExprOpts_DefaultAtoms
bondexpr = OEExprOpts_DefaultBonds
mcss.Init(refmol, atomexpr, bondexpr)
mcss.SetMCSFunc(OEMCSMaxBondsCompleteCycles())

image = OEImage(400, 200)

rows = 1
cols = 2
grid = OEImageGrid(image, rows, cols)
refcell = grid.GetCell(1, 1)
fitcell = grid.GetCell(1, 2)

opts = OE2DMolDisplayOptions(grid.GetCellWidth(), grid.GetCellHeight(), OEScale_AutoScale)
opts.SetTitleLocation(OETitleLocation_Hidden)

refscale = OEGetMoleculeScale(refmol, opts)
fitscale = OEGetMoleculeScale(fitmol, opts)
opts.SetScale(min(refscale, fitscale))

hstyle = OEHighlightStyle_BallAndStick

unique = True
miter = mcss.Match(fitmol, unique)
if miter.IsValid():
    match = miter.Target()
    OEPrepareAlignedDepiction(fitmol, mcss.GetPattern(), match)

    # depict reference molecule with MCS highlighting
    refdisp = OE2DMolDisplay(mcss.GetPattern(), opts)
    matchedatoms = OEIsAtomMember(match.GetPatternAtoms())
    matchedbonds = OEIsBondMember(match.GetPatternBonds())
    OEAddHighlighting(refdisp, OEPinkTint, hstyle, OENotAtom(matchedatoms), OENotBond(matchedbonds))

    # abset = OEAtomBondSet(match.GetPatternAtoms(), match.GetPatternBonds())
    # OEAddHighlighting(refdisp, OEBlueTint, hstyle, abset)
    OERenderMolecule(refcell, refdisp)

    # depict fit molecule with MCS highlighting
    fitdisp = OE2DMolDisplay(fitmol, opts)

    matchedatoms = OEIsAtomMember(match.GetTargetAtoms())
    matchedbonds = OEIsBondMember(match.GetTargetBonds())
    OEAddHighlighting(fitdisp, OEPinkTint, hstyle, OENotAtom(matchedatoms), OENotBond(matchedbonds))

    # abset = OEAtomBondSet(match.GetTargetAtoms(), match.GetTargetBonds())
    # OEAddHighlighting(fitdisp, OEBlueTint, hstyle, abset)
    OERenderMolecule(fitcell, fitdisp)

OEWriteImage("modMCSAlign.png", image)



##
# File:    OeShapeSearch.py
# Author:  J. Westbrook
# Date:    24-Jan-2012
# Version: 0.001
#
# Updated:
#    6-Jun-2016 jdw general cleanup
#
#
##
"""
Grid based shape search including chemical property coloring.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import sys

from openeye.oeshape import OEBestOverlay, OEBestOverlayScoreIter, OECalcVolume, OEColorFFType_ImplicitMillsDean, OEHighestTanimotoCombo, OESortOverlayScores


class OeShapeSearch(object):
    def __init__(self, verbose=False, log=sys.stderr):
        self.__verbose = verbose
        self.__lfh = log
        self.__refMol = None
        self.__fitMol = None
        self.__bestSearch = None
        self.__scoreiter = None

    def setRefMol(self, refMol):
        self.__refMol = refMol
        self.__bestSearch = OEBestOverlay()
        self.__bestSearch.SetRefMol(self.__refMol)
        self.__bestSearch.SetColorForceField(OEColorFFType_ImplicitMillsDean)
        self.__bestSearch.SetColorOptimize(True)
        self.__scoreiter = OEBestOverlayScoreIter()

    def setFitMol(self, fitMol):
        self.__fitMol = fitMol
        OESortOverlayScores(self.__scoreiter, self.__bestSearch.Overlay(self.__fitMol), OEHighestTanimotoCombo())
        retD = {}
        rL = []
        for score in self.__scoreiter:
            sD = {}
            sD["tanimotoCombo"] = score.GetTanimotoCombo()
            sD["tanimoto"] = score.GetShapeTanimoto()
            sD["tanimotoColor"] = score.GetColorTanimoto()
            sD["fitVolume"] = OECalcVolume(self.__fitMol, False)
            sD["refVolume"] = OECalcVolume(self.__refMol, False)
            rL.append(sD)
        retD[fitMol.GetTitle()] = rL
        #
        return retD



##
# File:    OeBuildModelMol.py
# Author:  jdw
# Date:    28-Nov-2014
# Version: 0.001
#
# Updates:
#   6-Jun-2016  jdw  general cleanup
#
##
"""
Classes to build OE molecule objects from chemical component model instances.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import sys
import traceback

from mmcif_utils.chemcomp.PdbxChemCompModel import (PdbxChemCompModel,
                                                    PdbxChemCompModelAtom,
                                                    PdbxChemCompModelBond)
from mmcif_utils.chemcomp.PdbxChemCompModelIo import PdbxChemCompModelIo
from openeye.oechem import (OE3DToInternalStereo, OEAroModelOpenEye,
                            OEAssignAromaticFlags, OECreateCanSmiString,
                            OECreateInChI, OECreateInChIKey,
                            OECreateIsoSmiString, OEFindRingAtomsAndBonds,
                            OEFormat_OEB, OEMol, OEMolecularFormula,
                            OEPerceiveCIPStereo, OESuppressHydrogens,
                            OEWriteMolecule, oemolistream, oemolostream)


class OeBuildModelMol(object):
    ''' Utility methods for constructing OEMols from chemical component model instances.
    '''

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        # File system path to the chemical component dictionary definitions in (CVS checkout organization)
        #
        # Internal storage for current OE molecule
        self.__oeMol = None
        #
        # Component identifier
        #
        self.__ccId = None
        self.__modelId = None
        #
        # dictionary of element counts eD[atno]=count
        self.__eD = {}
        #
        # Source data categories objects from chemical component definitions.
        #
        self.__ccAtomDL = None
        self.__ccBondDL = None
        self.__molXyzL = []

    def setDebug(self, flag):
        self.__debug = flag

    def setChemCompModelPath(self, modelPath):
        try:
            ccm = PdbxChemCompModelIo(verbose=self.__verbose, log=self.__lfh)
            ccm.setFilePath(modelPath)
            ccDL = ccm.getAttribDictList(catName='pdbx_chem_comp_model')
            cmp = PdbxChemCompModel(ccDL[0], self.__verbose, self.__lfh)
            self.__modelId = cmp.getId()
            self.__ccId = cmp.getCompId()

            self.__ccAtomDL = ccm.getAttribDictList(catName='pdbx_chem_comp_model_atom')
            self.__ccBondDL = ccm.getAttribDictList(catName='pdbx_chem_comp_model_bond')
            return self.__modelId
        except Exception as e:
            self.__lfh.write("OeBuildModelMol(setChemCompModelPath) Fails for %s %s\n" % (modelPath, str(e)))
            traceback.print_exc(file=self.__lfh)
        return None

    def __clear(self):
        self.__oeMol = None
        self.__eD = {}

    def serialize(self):
        """ Create a string representing the content of the current OE molecule.   This
            serialization uses the OE internal binary format.
        """
        oms = oemolostream()
        oms.SetFormat(OEFormat_OEB)
        oms.openstring()
        OEWriteMolecule(oms, self.__oeMol)
        if (self.__debug):
            self.__lfh.write("OeBuildModelMol(Serialize) SMILES %s\n" % OECreateCanSmiString(self.__oeMol))
            self.__lfh.write("OeBuildModelMol(Serialize) atoms = %d\n" % self.__oeMol.NumAtoms())
        return oms.GetString()

    def deserialize(self, oeS):
        """ Reconstruct an OE molecule from the input string serialization (OE binary).

            The deserialized molecule is used to initialize the internal OE molecule
            within this object.

            Returns True for success or False otherwise.
        """
        self.__clear()
        ims = oemolistream()
        ims.SetFormat(OEFormat_OEB)
        ims.openstring(oeS)

        nmol = 0
        mList = []
        # for mol in ims.GetOEGraphMols():
        for mol in ims.GetOEMols():
            if (self.__debug):
                self.__lfh.write("OeBuildModelMol(deserialize) SMILES %s\n" % OECreateCanSmiString(mol))
                self.__lfh.write("OeBuildModelMol(deserialize) title  %s\n" % mol.GetTitle())
                self.__lfh.write("OeBuildModelMol(deserialize) atoms  %d\n" % mol.NumAtoms())
            # mList.append(OEGraphMol(mol))
            mList.append(OEMol(mol))
            nmol += 1
        #
        if nmol >= 1:
            self.__oeMol = mList[0]
            self.__ccId = self.__oeMol.GetTitle()
            #
            if (self.__debug):
                self.__lfh.write("OeBuildModelMol(deserialize) mols  %d\n" % nmol)
                self.__lfh.write("OeBuildModelMol(deserialize) id %s\n" % self.__ccId)
                self.__lfh.write("OeBuildModelMol(deserialize) atoms  %d\n" % self.__oeMol.NumAtoms())
            return True
        else:
            return False

    def getElementCounts(self):
        """ Get the dictionary of element counts (eg. eD[iAtNo]=iCount).
        """
        if len(self.__eD) == 0:
            # calculate from current oeMol
            try:
                self.__eD = {}
                for atom in self.__oeMol.GetAtoms():
                    atNo = atom.GetAtomicNum()
                    if atNo not in self.__eD:
                        self.__eD[atNo] = 1
                    else:
                        self.__eD[atNo] += 1
            except:  # noqa: E722 pylint: disable=bare-except
                pass

        return self.__eD

    def build3D(self):
        try:
            self.__build3D()
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def __build3D(self):
        """ Build OE molecule using model instance 3D coordinates and OE stereo perception.
        """
        self.__clear()
        self.__oeMol = OEMol()
        #
        self.__oeMol.SetTitle(self.__modelId)
        aL = []

        # Atom index dictionary
        aD = {}
        i = 1

        for d in self.__ccAtomDL:
            ccAt = PdbxChemCompModelAtom(d, self.__verbose, self.__lfh)
            atName = ccAt.getName()
            aD[atName] = i
            i += 1
            atNo = ccAt.getAtNo()
            if atNo not in self.__eD:
                self.__eD[atNo] = 1
            else:
                self.__eD[atNo] += 1

            atType = ccAt.getType()
            fc = ccAt.getFormalCharge()
            # chFlag=ccAt.isChiral()
            # arFlag=ccAt.isAromatic()
            isotope = ccAt.getIsotope()
            oeAt = self.__oeMol.NewAtom(atNo)
            oeAt.SetName(atName)
            oeAt.SetFormalCharge(fc)
            # oeAt.SetChiral(chFlag)
            oeAt.SetIsotope(isotope)

            # oeAt.SetAromatic(arFlag)
            # if chFlag:
            #    st=ccAt.getCIPStereo()
            #    if st == 'S' or st == 'R':
            #        oeAt.SetStringData("StereoInfo",st)
            # if (self.__debug):
            #    self.__lfh.write("Atom - %s type %s atno %d isotope %d fc %d chFlag %r\n" % (atName,atType,atNo,isotope,fc,chFlag))

            cTup = ccAt.getModelCoordinates()
            # if (self.__verbose):
            #    self.__lfh.write("CC %s Atom - %s cTup %r\n" % (self.__ccId,atName,cTup))
            self.__oeMol.SetCoords(oeAt, cTup)
            if (self.__debug):
                self.__lfh.write("Atom - %s type %s atno %d isotope %d fc %d (xyz) %r\n" % (atName, atType, atNo, isotope, fc, cTup))
            aL.append(oeAt)

        for d in self.__ccBondDL:
            ccBnd = PdbxChemCompModelBond(d, self.__verbose, self.__lfh)
            (at1, at2) = ccBnd.getBond()
            iat1 = aD[at1] - 1
            iat2 = aD[at2] - 1
            iType = ccBnd.getIntegerType()
            #
            # arFlag=ccBnd.isAromatic()
            if (self.__debug):
                self.__lfh.write(" %s %d -- %s %d (%d)\n" % (at1, iat1, at2, iat2, iType))

            oeBnd = self.__oeMol.NewBond(aL[iat1], aL[iat2], iType)  # noqa: F841 pylint: disable=unused-variable

            # oeBnd.SetAromatic(arFlag)
            # if arFlag:
            #    oeBnd.SetIntType(5)
            # st=ccBnd.getStereo()
            # if st == 'E' or st =='Z':
            #    oeBnd.SetStringData("StereoInfo",st)

        #
        # run standard perceptions --
        #
        self.__oeMol.SetDimension(3)
        OE3DToInternalStereo(self.__oeMol)
        OEFindRingAtomsAndBonds(self.__oeMol)
        # Other aromatic models: OEAroModelMDL or OEAroModelDaylight
        OEAssignAromaticFlags(self.__oeMol, OEAroModelOpenEye)
        self.updateCIPStereoOE()

    def updateCIPStereoOE(self):
        """ OE perception of CIP stereo -
        """
        for atom in self.__oeMol.GetAtoms():
            OEPerceiveCIPStereo(self.__oeMol, atom)

        for bond in self.__oeMol.GetBonds():
            if (bond.GetOrder() == 2):
                OEPerceiveCIPStereo(self.__oeMol, bond)

    def getGraphMolSuppressH(self):
        """ Return the current constructed OE molecule with hydrogens suppressed.
        """
        # OESuppressHydrogens(self.__oeMol, retainPolar=False,retainStereo=True,retainIsotope=True)
        OESuppressHydrogens(self.__oeMol)
        return self.__oeMol

    def getMol(self):
        """ Return the current constructed OE molecule.
        """
        return self.__oeMol

    def getCanSMILES(self):
        """ Return the cannonical SMILES string derived from the current OD molecule.
        """
        return OECreateCanSmiString(self.__oeMol)

    def getIsoSMILES(self):
        """ Return the cannonical stereo SMILES string derived from the current OE molecule.
        """
        return OECreateIsoSmiString(self.__oeMol)

    def getFormula(self):
        """ Return the Hill order formulat  derived from the current OE molecule.
        """
        return OEMolecularFormula(self.__oeMol)

    def getInChIKey(self):
        """ Return the InChI key derived from the current OE molecule.
        """
        return OECreateInChIKey(self.__oeMol)

    def getInChI(self):
        """ Return the InChI string derived from the current OE molecule.
        """
        return OECreateInChI(self.__oeMol)

    def getTitle(self):
        """ Return the title assigned to the current OE molecule
        """
        return self.__oeMol.GetTitle()

    def getCoords(self):
        """  Return coordinate list if a 3D molecule is built -- otherwise an empty list --

        """
        return self.__molXyzL



##
# File:    simple-depict-example.py
# Author:  jdw
# Date:    7-Sep-2013
# Version: 0.001
#
# Updates:
#      6-Jun-2016 jdw general cleanup -
##
"""
An example of generating 2D chemical diagrams in svg format from chemical component definition.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import sys
import traceback
import unittest

from wwpdb.utils.oe_util.build.OeChemCompIoUtils import OeChemCompIoUtils
from wwpdb.utils.oe_util.oedepict.OeDepict import OeDepict


class OeDepictTests(unittest.TestCase):
    def setUp(self):
        self.__lfh = sys.stderr
        self.__verbose = True
        # ---------------------------------------------------------------------------------
        # The following needs to be set to the path of a checked out copy of the CVS ligand repository.
        ###
        self.__topCachePath = "/data/components/ligand-dict-v3"
        ##
        # example list of 3-letter-codes -
        ##
        self.__idList = ["atp", "gtp", "A", "C", "G", "DG"]

    def tearDown(self):
        pass

    def testDepictIdList(self):
        """Test case -  get, read, build OE molecule, and depict the molecule."""
        self.__lfh.write("\nStarting OeDepictTests testDepictIDList\n")
        try:
            oeMolTitleList = self.__makeFromIdList(idList=self.__idList)
            if self.__verbose:
                self.__lfh.write("molTitleList length is %d\n" % len(oeMolTitleList))
            #
            for ccId, mol, title in oeMolTitleList:
                imagePath = ccId + ".svg"
                oed = OeDepict(verbose=self.__verbose, log=self.__lfh)
                oed.setMolTitleList([(ccId, mol, title)])
                oed.setDisplayOptions(imageX=250, imageY=250, labelAtomName=True, labelAtomCIPStereo=True, labelAtomIndex=False, labelBondIndex=False, bondDisplayWidth=0.5)
                oed.setGridOptions(rows=1, cols=1)
                oed.prepare()
                oed.write(imagePath)
        except:  # noqa: E722 pylint: disable=bare-except
            traceback.print_exc(file=self.__lfh)
            self.fail()

    def __makeFromIdList(self, idList):
        """Create OE molecules from the input chemical component definition id list."""
        self.__lfh.write("\nStarting OeDepictTests __makeFromIdList\n")
        try:
            oemList = []
            oeU = OeChemCompIoUtils(topCachePath=self.__topCachePath, verbose=self.__verbose, log=self.__lfh)
            oemList = oeU.getFromIdList(idList, use3D=True, coordType="model")
            for oem in oemList:
                self.__lfh.write("Title              = %s\n" % oem.getTitle())
                self.__lfh.write("SMILES (canonical) = %s\n" % oem.getCanSMILES())
                self.__lfh.write("SMILES (isomeric)  = %s\n" % oem.getIsoSMILES())
            return zip(idList, oemList, idList)
        except:  # noqa: E722 pylint: disable=bare-except
            traceback.print_exc(file=self.__lfh)
            self.fail()


def suiteDepict():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(OeDepictTests("testDepictIdList"))
    return suiteSelect


if __name__ == "__main__":
    mySuite2 = suiteDepict()
    unittest.TextTestRunner(verbosity=2).run(mySuite2)
    #



# File: setup.py
# Date: 6-Oct-2018
#
# Update:
#
import re

from setuptools import find_packages
from setuptools import setup

packages = []
thisPackage = "wwpdb.utils.oe_util"

with open("wwpdb/utils/oe_util/__init__.py", "r") as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError("Cannot find version information")


setup(
    name=thisPackage,
    version=version,
    description="wwPDB workflow engine utils",
    long_description="See:  README.md",
    author="Ezra Peisach",
    author_email="ezra.peisach@rcsb.org",
    url="https://github.com/rcsb/py-wwpdb_apps_wf_engine_utils",
    #
    license="Apache 2.0",
    classifiers=[
        "Development Status :: 3 - Alpha",
        # 'Development Status :: 5 - Production/Stable',
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    entry_points={"console_scripts": []},
    #
    # Also depends on 'openeye.oechem' but cannot install by pypi
    install_requires=[
        "mmcif.utils",
        "wwpdb.utils.cc_dict_util ~= 0.2",
        "OpenEye-toolkits-python2.7-ucs2-linux-x64 == 2016.6.1; python_version == '2.7' and sys_platform=='linux2'",
        "OpenEye-toolkits ; python_version >= '3'",
    ],
    packages=find_packages(exclude=["wwpdb.utils.tests_oe_util"]),
    package_data={
        # If any package contains *.md or *.rst ...  files, include them:
        "": ["*.md", "*.rst", "*.txt", "*.cfg"]
    },
    #
    # These basic tests require no database services -
    test_suite="wwpdb.utils.tests_oe_util",
    tests_require=["tox"],
    #
    # Not configured ...
    extras_require={
        "dev": ["check-manifest"],
        "test": ["coverage"],
    },
    # Added for
    command_options={"build_sphinx": {"project": ("setup.py", thisPackage), "version": ("setup.py", version), "release": ("setup.py", version)}},
    # This setting for namespace package support -
    zip_safe=False,
)



##
# File:    OeChemCompIoUtils.py
# Author:  jdw
# Date:    04-May-2013
# Version: 0.001
#
# Updates:
#      6-Jun-2016  jdw  general cleanup - use native Python parser on darwin
##
"""
Construct OE mol data structures from chemical component definitions.  Methods are provided
provided to manage the chemical component definition IO

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"

import os
import os.path
import sys
import traceback

from wwpdb.utils.oe_util.build.OeBuildMol import OeBuildMol

#
#  Storage adapater can be changed here --
if sys.platform in ['darwin']:
    from mmcif_utils.persist.PdbxPyIoAdapter import \
        PdbxPyIoAdapter as PdbxIoAdapter
else:
    from mmcif_utils.persist.PdbxPyIoAdapter import \
        PdbxPyIoAdapter as PdbxIoAdapter


class OeChemCompIoUtils(object):
    ''' Construct OE mol data structures from chemical component definitions.  Methods are provided
        provided to manage the chemical component definition IO
    '''

    def __init__(self, topCachePath='/data/components/ligand-dict-v3', verbose=True, log=sys.stderr):
        self.__topCachePath = topCachePath
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #

    def getFromPathList(self, pathList, use3D=True, coordType="model", setTitle=True):
        """ Return a list of OE mols constructed from the input pathList of chemical definitions.
        """
        oemList = []
        try:
            for pth in pathList:
                myReader = PdbxIoAdapter(self.__verbose, self.__lfh)
                ok = myReader.read(pdbxFilePath=pth)  # noqa: F841 pylint: disable=unused-variable
                for container in myReader.getContainerList():
                    oem = OeBuildMol(verbose=self.__verbose, log=self.__lfh)
                    oem.setDebug(self.__debug)
                    oem.set(container.getName(),
                            dcChemCompAtom=container.getObj("chem_comp_atom"),
                            dcChemCompBond=container.getObj("chem_comp_bond"))
                    if use3D:
                        oem.build3D(coordType=coordType, setTitle=setTitle)
                    else:
                        oem.build2D(setTitle=setTitle)
                    #
                    oemList.append(oem)
                    #
                    if self.__debug:
                        self.__lfh.write("+OeChemCompIoUtils.getOeMols() Title              = %s\n" % oem.getTitle())
                        self.__lfh.write("+OeChemCompIoUtils.getOeMols() SMILES (canonical) = %s\n" % oem.getCanSMILES())
                        self.__lfh.write("+OeChemCompIoUtils.getOeMols() SMILES (isomeric)  = %s\n" % oem.getIsoSMILES())
        except Exception as e:
            if self.__verbose:
                self.__lfh.write("+OeChemCompIoUtils.getOeMols() Failed %s\n" % str(e))
                traceback.print_exc(file=self.__lfh)
        return oemList

    def getFromIdList(self, idList, use3D=True, coordType="model", setTitle=True):
        """ Return a list of OE mols constructed from the input Id of chemical definitions.
        """
        pathList = []
        for ide in idList:
            if len(ide) < 1:
                continue
            idU = str(ide).upper()
            if idU.startswith("PRDCC_"):
                hashd = idU[-1]
                pth = os.path.join(self.__topCachePath, hashd, idU + '.cif')
            else:
                hashd = idU[0]
                pth = os.path.join(self.__topCachePath, hashd, idU, idU + '.cif')
            pathList.append(pth)
        return self.getFromPathList(pathList, use3D=use3D, coordType=coordType, setTitle=setTitle)



##
# File:    OeBuildMol.py
# Author:  jdw
# Date:    25-Sep-2011
# Version: 0.001
#
# Updates:
#  1-Oct-2011 jdw Add 2D builder, CIP Stereo and SMILES
# 18-Feb-2012 jdw add 3D builder with aromatic and CIP perception.
# 20-Feb-2012 jdw Add serialization and deserialization
# 20-Feb-2012 jdw Add CIP perception for 3D builder
# 23-Feb-2012 jdw Moved to oe_util/build path.
#                 Revised to use chemical component iterator classes
#                 and to support flat file and persist storage models.
#  5-May-2014 jdw add method build molecule from input SMILES importSmiles(smiles)
#  6-Jun-2016 jdw general cleanup and platform selection parsing -  use native python on darwin
#  8-Jun-2016 jdw add method to instantiate using an existing OEMOL and method to return ccId.
# 31-Aug-2020 ZF  in importSmiles() function, added checking OEParseSmiles() return status
#
##
"""
Classes to build OE molecule objects from chemical component definition data.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Creative Commons Attribution 3.0 Unported"
__version__ = "V0.01"


import sys
import traceback

from mmcif.io.IoAdapterCore import IoAdapterCore as IoAdapter
# from mmcif.api.PdbxContainers import *
from mmcif_utils.chemcomp.PdbxChemComp import PdbxChemCompConstants
from openeye.oechem import (OE3DToInternalStereo, OEAddExplicitHydrogens,
                            OEAroModelOpenEye, OEAssignAromaticFlags,
                            OECIPAtomStereo_R, OECIPAtomStereo_S,
                            OECIPBondStereo_E, OECIPBondStereo_Z,
                            OECreateCanSmiString, OECreateInChI,
                            OECreateInChIKey, OECreateIsoSmiString,
                            OEFindRingAtomsAndBonds, OEFloatArray,
                            OEFormat_OEB, OEGetAtomicSymbol, OEGraphMol, OEMol,
                            OEMolecularFormula, OEParseSmiles,
                            OEPerceiveChiral, OEPerceiveCIPStereo,
                            OEReadMolecule, OESetCIPStereo,
                            OESuppressHydrogens, OETriposAtomNames,
                            OEWriteMolecule, oemolistream, oemolostream)
from wwpdb.utils.cc_dict_util.persist.PdbxChemCompPersist import (
    PdbxChemCompAtomIt, PdbxChemCompBondIt)


class OeBuildMol(object):
    ''' Utility methods for constructing OEGraphMols from chemical component definition objects.
    '''

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        # File system path to the chemical component dictionary definitions in (CVS checkout organization)
        #
        # Internal storage for current OE molecule
        self.__oeMol = None
        #
        # Component identifier
        #
        self.__ccId = None
        #
        # dictionary of element counts eD[atno]=count
        self.__eD = {}
        #
        # Source data categories objects from chemical component definitions.
        self.__dcChemCompAtom = None
        self.__dcChemCompBond = None
        #
        self.__molXyzL = []

    def setDebug(self, flag):
        self.__debug = flag

    def setChemCompPath(self, ccPath):
        try:
            myReader = IoAdapter(self.__verbose, self.__lfh)
            cL = myReader.readFile(ccPath)
            self.__ccId = cL[0].getName()
            self.__dcChemCompAtom = cL[0].getObj("chem_comp_atom")
            self.__dcChemCompBond = cL[0].getObj("chem_comp_bond")
            return self.__ccId
        except Exception as e:
            self.__lfh.write("OeBuildMol(setChemCompPath) Fails for %s %s\n" % (ccPath, str(e)))
            traceback.print_exc(file=self.__lfh)
        return None

    def setOeMol(self, inpOeMol, ccId):
        """  Load this object with an existing oeMOL()
        """
        self.__clear()
        self.__oeMol = OEMol(inpOeMol)
        self.__ccId = ccId
        self.getElementCounts()

    def set(self, ccId, dcChemCompAtom=None, dcChemCompBond=None):
        """  Assign source data categories -
        """
        self.__ccId = ccId
        self.__dcChemCompAtom = dcChemCompAtom
        self.__dcChemCompBond = dcChemCompBond

    def __clear(self):
        self.__oeMol = None
        self.__eD = {}

    def serialize(self):
        """ Create a string representing the content of the current OE molecule.   This
            serialization uses the OE internal binary format.
        """
        oms = oemolostream()
        oms.SetFormat(OEFormat_OEB)
        oms.openstring()
        OEWriteMolecule(oms, self.__oeMol)
        if (self.__debug):
            self.__lfh.write("OeBuildMol(Serialize) SMILES %s\n" % OECreateCanSmiString(self.__oeMol))
            self.__lfh.write("OeBuildMol(Serialize) atoms = %d\n" % self.__oeMol.NumAtoms())
        return oms.GetString()

    def deserialize(self, oeS):
        """ Reconstruct an OE molecule from the input string serialization (OE binary).

            The deserialized molecule is used to initialize the internal OE molecule
            within this object.

            Returns True for success or False otherwise.
        """
        self.__clear()
        ims = oemolistream()
        ims.SetFormat(OEFormat_OEB)
        ims.openstring(oeS)

        nmol = 0
        mList = []
        # for mol in ims.GetOEGraphMols():
        for mol in ims.GetOEMols():
            if (self.__debug):
                self.__lfh.write("OeBuildMol(deserialize) SMILES %s\n" % OECreateCanSmiString(mol))
                self.__lfh.write("OeBuildMol(deserialize) title  %s\n" % mol.GetTitle())
                self.__lfh.write("OeBuildMol(deserialize) atoms  %d\n" % mol.NumAtoms())
            # mList.append(OEGraphMol(mol))
            mList.append(OEMol(mol))
            nmol += 1
        #
        if nmol >= 1:
            self.__oeMol = mList[0]
            self.__ccId = self.__oeMol.GetTitle()
            #
            if (self.__debug):
                self.__lfh.write("OeBuildMol(deserialize) mols  %d\n" % nmol)
                self.__lfh.write("OeBuildMol(deserialize) id %s\n" % self.__ccId)
                self.__lfh.write("OeBuildMol(deserialize) atoms  %d\n" % self.__oeMol.NumAtoms())
            return True
        else:
            return False

    def simpleAtomNames(self):
        """
        """
        for atom in self.__oeMol.GetAtoms():
            atom.SetIntType(atom.GetAtomicNum())
            atom.SetType(OEGetAtomicSymbol(atom.GetAtomicNum()))
        OETriposAtomNames(self.__oeMol)

    def getElementCounts(self):
        """ Get the dictionary of element counts (eg. eD[iAtNo]=iCount).
        """
        if len(self.__eD) == 0:
            # calculate from current oeMol
            try:
                self.__eD = {}
                for atom in self.__oeMol.GetAtoms():
                    atNo = atom.GetAtomicNum()
                    if atNo not in self.__eD:
                        self.__eD[atNo] = 1
                    else:
                        self.__eD[atNo] += 1
            except:  # noqa: E722 pylint: disable=bare-except
                pass

        return self.__eD

    def build3D(self, coordType="model", setTitle=True):
        try:
            self.__build3D(coordType=coordType, setTitle=setTitle)
            return True
        except Exception as e:
            self.__lfh.write("OeBuildMol(build3D) Failing %s\n" % str(e))
            traceback.print_exc(file=self.__lfh)
        return False

    def __build3D(self, coordType="model", setTitle=True):
        """ Build OE molecule using 3D coordinates and OE stereo perception.
        """
        self.__clear()
        # self.__oeMol=OEGraphMol()
        self.__oeMol = OEMol()
        #
        if setTitle:
            self.__oeMol.SetTitle(self.__ccId)
        aL = []

        # Atom index dictionary
        aD = {}
        i = 1

        atomIt = PdbxChemCompAtomIt(self.__dcChemCompAtom, self.__verbose, self.__lfh)
        for ccAt in atomIt:

            atName = ccAt.getName()
            aD[atName] = i
            i += 1
            atNo = ccAt.getAtNo()
            if atNo not in self.__eD:
                self.__eD[atNo] = 1
            else:
                self.__eD[atNo] += 1

            atType = ccAt.getType()
            fc = ccAt.getFormalCharge()
            chFlag = ccAt.isChiral()
            # arFlag = ccAt.isAromatic()
            isotope = ccAt.getIsotope()
            leavingAtom = ccAt.getLeavingAtomFlag()

            oeAt = self.__oeMol.NewAtom(atNo)
            oeAt.SetName(atName)
            oeAt.SetFormalCharge(fc)
            oeAt.SetStringData("pdbx_leaving_atom_flag", leavingAtom)
            oeAt.SetChiral(chFlag)
            oeAt.SetIsotope(isotope)

            # oeAt.SetAromatic(arFlag)
            # if chFlag:
            #    st=ccAt.getCIPStereo()
            #    if st == 'S' or st == 'R':
            #        oeAt.SetStringData("StereoInfo",st)
            # if (self.__debug):
            #    self.__lfh.write("Atom - %s type %s atno %d isotope %d fc %d chFlag %r\n" % (atName,atType,atNo,isotope,fc,chFlag))

            if ((coordType == 'model') and ccAt.hasModelCoordinates()):
                cTup = ccAt.getModelCoordinates()
                # if (self.__verbose):
                #    self.__lfh.write("CC %s Atom - %s cTup %r\n" % (self.__ccId,atName,cTup))
                self.__oeMol.SetCoords(oeAt, cTup)
            elif ((coordType == 'ideal') and ccAt.hasIdealCoordinates()):
                cTup = ccAt.getIdealCoordinates()
                self.__oeMol.SetCoords(oeAt, cTup)
            else:
                pass

            if (self.__debug):
                self.__lfh.write("Atom - %s type %s atno %d isotope %d fc %d (xyz) %r\n" % (atName, atType, atNo, isotope, fc, cTup))
            aL.append(oeAt)

        bondIt = PdbxChemCompBondIt(self.__dcChemCompBond, self.__verbose, self.__lfh)
        for ccBnd in bondIt:
            (at1, at2) = ccBnd.getBond()
            iat1 = aD[at1] - 1
            iat2 = aD[at2] - 1
            iType = ccBnd.getIntegerType()
            #
            arFlag = ccBnd.isAromatic()  # noqa: F841 pylint: disable=unused-variable
            if (self.__debug):
                self.__lfh.write(" %s %d -- %s %d (%d)\n" % (at1, iat1, at2, iat2, iType))

            oeBnd = self.__oeMol.NewBond(aL[iat1], aL[iat2], iType)  # noqa: F841 pylint: disable=unused-variable

            # oeBnd.SetAromatic(arFlag)
            # if arFlag:
            #    oeBnd.SetIntType(5)
            # st=ccBnd.getStereo()
            # if st == 'E' or st =='Z':
            #    oeBnd.SetStringData("StereoInfo",st)

        #
        # run standard perceptions --
        #
        self.__oeMol.SetDimension(3)
        OE3DToInternalStereo(self.__oeMol)
        OEFindRingAtomsAndBonds(self.__oeMol)
        # Other aromatic models: OEAroModelMDL or OEAroModelDaylight
        OEAssignAromaticFlags(self.__oeMol, OEAroModelOpenEye)
        self.updateCIPStereoOE()

    def updatePerceptions3D(self):
        self.__oeMol.SetDimension(3)
        OE3DToInternalStereo(self.__oeMol)
        OEFindRingAtomsAndBonds(self.__oeMol)
        # Other aromatic models: OEAroModelMDL or OEAroModelDaylight
        OEAssignAromaticFlags(self.__oeMol, OEAroModelOpenEye)
        self.updateCIPStereoOE()

    def updateCIPStereoOE(self):
        """ OE perception of CIP stereo -
        """
        for atom in self.__oeMol.GetAtoms():
            OEPerceiveCIPStereo(self.__oeMol, atom)

        for bond in self.__oeMol.GetBonds():
            if (bond.GetOrder() == 2):
                OEPerceiveCIPStereo(self.__oeMol, bond)

    def build2D(self, setTitle=True):  # pylint: disable=unused-argument
        try:
            self.__build2D(setTitle=True)
            return True
        except:  # noqa: E722 pylint: disable=bare-except
            return False

    def __build2D(self, setTitle=True):
        """  Build molecule using existing assignments of chemical information in the CC definition.
        """
        self.__clear()
        self.__oeMol = OEGraphMol()
        if setTitle:
            self.__oeMol.SetTitle(self.__ccId)
        aL = []
        i = 1
        # Atom index dictionary
        aD = {}

        atomIt = PdbxChemCompAtomIt(self.__dcChemCompAtom, self.__verbose, self.__lfh)
        for ccAt in atomIt:
            atName = ccAt.getName()
            aD[atName] = i
            i += 1
            atNo = ccAt.getAtNo()
            if atNo not in self.__eD:
                self.__eD[atNo] = 1
            else:
                self.__eD[atNo] += 1
            atType = ccAt.getType()
            fc = ccAt.getFormalCharge()
            chFlag = ccAt.isChiral()
            arFlag = ccAt.isAromatic()
            isotope = ccAt.getIsotope()
            leavingAtom = ccAt.getLeavingAtomFlag()

            oeAt = self.__oeMol.NewAtom(atNo)
            oeAt.SetName(atName)
            oeAt.SetFormalCharge(fc)
            oeAt.SetStringData("pdbx_leaving_atom_flag", leavingAtom)
            oeAt.SetChiral(chFlag)
            oeAt.SetIsotope(isotope)
            oeAt.SetAromatic(arFlag)
            if chFlag:
                st = ccAt.getCIPStereo()
                if st == 'S' or st == 'R':
                    oeAt.SetStringData("StereoInfo", st)
            if (self.__debug):
                self.__lfh.write("Atom - %s type %s atno %d isotope %d fc %d chFlag %r\n" % (atName, atType, atNo, isotope, fc, chFlag))
            aL.append(oeAt)

        bondIt = PdbxChemCompBondIt(self.__dcChemCompBond, self.__verbose, self.__lfh)
        for ccBnd in bondIt:
            (at1, at2) = ccBnd.getBond()
            iat1 = aD[at1] - 1
            iat2 = aD[at2] - 1
            iType = ccBnd.getIntegerType()
            arFlag = ccBnd.isAromatic()
            if (self.__debug):
                self.__lfh.write(" %s %d -- %s %d (%d)\n" % (at1, iat1, at2, iat2, iType))

            oeBnd = self.__oeMol.NewBond(aL[iat1], aL[iat2], iType)
            oeBnd.SetAromatic(arFlag)
            if arFlag:
                oeBnd.SetIntType(5)
            st = ccBnd.getStereo()
            if st == 'E' or st == 'Z':
                oeBnd.SetStringData("StereoInfo", st)

        #
        # run standard perceptions --
        OEFindRingAtomsAndBonds(self.__oeMol)
        OEPerceiveChiral(self.__oeMol)

        for oeAt in self.__oeMol.GetAtoms():
            st = oeAt.GetStringData("StereoInfo")
            if st == 'R':
                OESetCIPStereo(self.__oeMol, oeAt, OECIPAtomStereo_R)
            elif st == 'S':
                OESetCIPStereo(self.__oeMol, oeAt, OECIPAtomStereo_S)

        for oeBnd in self.__oeMol.GetBonds():
            st = oeBnd.GetStringData("StereoInfo")
            if st == 'E':
                OESetCIPStereo(self.__oeMol, oeBnd, OECIPBondStereo_E)
            elif st == 'Z':
                OESetCIPStereo(self.__oeMol, oeBnd, OECIPBondStereo_Z)
        if (self.__debug):
            for ii, atm in enumerate(self.__oeMol.GetAtoms()):
                self.__lfh.write("OeBuildMol.build2d - atom  %d %s\n" % (ii, atm.GetName()))

    def importFile(self, filePath, type='2D'):  # pylint: disable=redefined-builtin
        """  Contruct a OEGraphMol using the content of the input file.  The input
             file must have a file extension recognized by the OE toolkit (e.g. .sdf)
        """
        ifs = oemolistream()
        if not ifs.open(filePath):
            return False
        #
        # self.__oeMol = OEGraphMol()
        self.__oeMol = OEMol()
        OEReadMolecule(ifs, self.__oeMol)
        #        OETriposAtomNames(self.__oeMol)
        if type == '2D':
            # run standard perceptions --
            OEFindRingAtomsAndBonds(self.__oeMol)
            OEPerceiveChiral(self.__oeMol)

            for oeAt in self.__oeMol.GetAtoms():
                st = oeAt.GetStringData("StereoInfo")
                if st == 'R':
                    OESetCIPStereo(self.__oeMol, oeAt, OECIPAtomStereo_R)
                elif st == 'S':
                    OESetCIPStereo(self.__oeMol, oeAt, OECIPAtomStereo_S)

            for oeBnd in self.__oeMol.GetBonds():
                st = oeBnd.GetStringData("StereoInfo")
                if st == 'E':
                    OESetCIPStereo(self.__oeMol, oeBnd, OECIPBondStereo_E)
                elif st == 'Z':
                    OESetCIPStereo(self.__oeMol, oeBnd, OECIPBondStereo_Z)
        elif type == '3D':
            # run standard perceptions --
            #
            self.__oeMol.SetDimension(3)
            OE3DToInternalStereo(self.__oeMol)
            OEFindRingAtomsAndBonds(self.__oeMol)
            # Other aromatic models: OEAroModelMDL or OEAroModelDaylight
            OEAssignAromaticFlags(self.__oeMol, OEAroModelOpenEye)
            self.updateCIPStereoOE()
            OEAddExplicitHydrogens(self.__oeMol)

        self.__molXyzL = []
        aC = {}
        for ii, atm in enumerate(self.__oeMol.GetAtoms()):
            iAtNum = atm.GetAtomicNum()
            if iAtNum in aC:
                aC[iAtNum] += 1
            else:
                aC[iAtNum] = 1
            # Less than idea - should have an API
            atName = PdbxChemCompConstants._periodicTable[iAtNum - 1] + str(aC[iAtNum])  # pylint: disable=protected-access
            atm.SetName(atName)
            #
            xyzL = OEFloatArray(3)
            self.__oeMol.GetCoords(atm, xyzL)
            self.__molXyzL.append((ii, atm.GetIdx(), atm.GetAtomicNum(), atm.GetName(), atm.GetType(), xyzL[0], xyzL[1], xyzL[2]))

        return True

    def importSmiles(self, smiles):
        """  Contruct a OEGraphMol using the input descriptor.
        """
        self.__oeMol = OEGraphMol()
        if OEParseSmiles(self.__oeMol, smiles):
            OEFindRingAtomsAndBonds(self.__oeMol)
            OEPerceiveChiral(self.__oeMol)
            return True
        #
        return False

    def getGraphMolSuppressH(self):
        """ Return the current constructed OE molecule with hydrogens suppressed.
        """
        # OESuppressHydrogens(self.__oeMol, retainPolar=False,retainStereo=True,retainIsotope=True)
        OESuppressHydrogens(self.__oeMol)
        return self.__oeMol

    def getMol(self):
        """ Return the current constructed OE molecule.
        """
        return OEMol(self.__oeMol)

    def getCanSMILES(self):
        """ Return the cannonical SMILES string derived from the current OD molecule.
        """
        return OECreateCanSmiString(self.__oeMol)

    def getIsoSMILES(self):
        """ Return the cannonical stereo SMILES string derived from the current OE molecule.
        """
        return OECreateIsoSmiString(self.__oeMol)

    def getFormula(self):
        """ Return the Hill order formulat  derived from the current OE molecule.
        """
        return OEMolecularFormula(self.__oeMol)

    def getInChIKey(self):
        """ Return the InChI key derived from the current OE molecule.
        """
        return OECreateInChIKey(self.__oeMol)

    def getInChI(self):
        """ Return the InChI string derived from the current OE molecule.
        """
        return OECreateInChI(self.__oeMol)

    def getTitle(self):
        """ Return the title assigned to the current OE molecule
        """
        return self.__oeMol.GetTitle()

    def getCcId(self):
        """ Return the CC id of this object -
        """
        return self.__ccId

    def getCoords(self):
        """  Return coordinate list if a 3D molecule is built -- otherwise an empty list --

        """
        return self.__molXyzL
